Object addSubclass: #StanfordBenchmarks
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
    category: 'Self-Benchmarks'
!

"*****************************************************************************"
"****************************** Benchmark Runner *****************************"
"*****************************************************************************"

StanfordBenchmarks addSubclass: #BenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: 'PrimarySuite FullSuite LowSuite TimesToRun'
	poolDictionaries: ''
	category: 'Self-Benchmarks'
 "library: Environment@#UserLib@#Benchmarks"
!


!BenchmarkRunner someMethodsFor: 'running'!

run
	self runSuite: PrimarySuite!

runFull
	self runSuite: FullSuite!

runLow
	self runSuite: LowSuite!

runAll
	| allTests |
	allTests := FullSuite, LowSuite.
	self runSuite: allTests!

runSuite: suite
	 "BenchmarkRunner new run"
	 "Run method updated by Greg Gritton"
	 | total time times bm |
	 total := 0.
	 suite do: 
	 	[:b | 
	 	 bm := (Smalltalk at: b) new.
	 	 times := TimesToRun at: b ifAbsent: [ 10 ].
	 	 time := Time millisecondsToRun: [ times timesRepeat: [bm run]].
	 	 total := total + time.
	 	 Transcript show: b asString, ': ' , times printString, ' runs at ' , (time // times) printString , ' ms.'.
	 	 Transcript cr].
	 Transcript cr.
	 Transcript show: 'Total: ' , total printString , ' ms.'.
	 Transcript cr!

oldRun
	 "BenchmarkRunner new oldRun"
	 "Original run task"
	 | total fixed float size time bm |
	 total := 0.
	 fixed := 0.
	 float := 0.
	 FullSuite do: 
	 	[:b | 
	 	 bm := (Smalltalk at: b) new.
	 	 time := Time millisecondsToRun: [bm run].
	 	 total := total + time.
	 	 fixed := fixed + (time * bm fixedBase).
	 	 float := float + (time * bm floatBase).
	 	 Transcript show: b asString, ': ' , time printString , ' ms.'.
	 	 Transcript cr].
	 Transcript cr.
	 Transcript show: 'Total: ' , total printString , ' ms.'.

	 size := FullSuite size asFloat.
	 fixed := fixed / size.
	 float := float / size.
	 Transcript show: 'Fixed:  ' , fixed printString , '.'.
	 Transcript cr.
	 Transcript show: 'Float:  ' , float printString , '.'.
	 Transcript cr!

runOne
	"BenchmarkRunner initialize. BenchmarkRunner new runOne"
	| time bm |
	FullSuite do: 
		[:b | 
		bm := (Smalltalk at: b) new.
		time := Time millisecondsToRun: [bm run].
		Transcript show: b asString, ': ' , time printString , ' ms'.
		Transcript cr].
 	Transcript cr!

runTen
	"BenchmarkRunner new runTen"

	| time bm |
	FullSuite do: 
		[:b | 
		bm := (Smalltalk at: b) new.
		time := Time millisecondsToRun: [10 timesRepeat: [bm run]].
	 	 Transcript show: b asString, ': ' , 10 printString, ' runs at ' , (time // 10) printString , ' ms.'.
		Transcript cr].
	Transcript cr!

runTests
	"BenchmarkRunner initialize. BenchmarkRunner new runTests"

	| ctime rtime bm |
	FullSuite do: 
		[:b | 
		bm := (Smalltalk at: b) new.
		Smalltalk garbageCollect.
		ctime := Time millisecondsToRun: [10 timesRepeat: [bm run]].
		Smalltalk garbageCollect.
		rtime := Time millisecondsToRun: [10 timesRepeat: [bm run]].
		ctime := ctime - rtime.
		rtime := rtime / 10.
		Transcript show: b asString, ': ' , ctime printString, ' ms (compile), ', rtime printString , ' ms (run)'.
		Transcript cr].
	Transcript cr! !


!BenchmarkRunner class someMethodsFor: 'initialization'!

initialize
	PrimarySuite := OrderedCollection new.
	FullSuite := OrderedCollection new.
	LowSuite := OrderedCollection new.
	PrimarySuite addAll: #(BubbleSortBenchmark IntMMBenchmark MMBenchmark
	                       PermBenchmark QueensBenchmark QuicksortBenchmark
	                       TreeSortBenchmark TowersBenchmark 
	                       FFTBenchmark PuzzleBenchmark).
	FullSuite addAll: #(BubbleSortBenchmark BubbleSort2Benchmark 
	                    IntMMBenchmark IntMM2Benchmark 
	                    MMBenchmark MM2Benchmark 
	                    PermBenchmark Perm2Benchmark 
	                    QueensBenchmark Queens2Benchmark 
	                    QuicksortBenchmark Quicksort2Benchmark
	                    TreeSortBenchmark TreeSort2Benchmark 
	                    TowersBenchmark Towers2Benchmark 
	                    FFTBenchmark PuzzleBenchmark).
	LowSuite addAll: #(SieveBenchmark SumToBenchmark FastSumToBenchmark 
	                   SumFromToBenchmark RecurseBenchmark AtAllPutBenchmark 
	                   SumAllBenchmark IncrementAllBenchmark NestedLoopBenchmark   
	                   TakBenchmark TaklBenchmark TreeSortBenchmark TreeSort2Benchmark).
	
	TimesToRun := Dictionary new.
	
	"PrimarySuite"
	TimesToRun at: #BubbleSortBenchmark  put: 10.
	TimesToRun at: #IntMMBenchmark       put: 10.
	TimesToRun at: #MMBenchmark          put: 10.
	TimesToRun at: #PermBenchmark        put: 10.
	TimesToRun at: #QueensBenchmark      put: 10.
	TimesToRun at: #QuicksortBenchmark   put: 10.
	TimesToRun at: #TowersBenchmark      put: 10.
	TimesToRun at: #TreeSortBenchmark    put: 10.
	TimesToRun at: #FFTBenchmark         put: 5.
	TimesToRun at: #PuzzleBenchmark      put: 3.

	"Additional tests in FullSuite"
	TimesToRun at: #BubbleSort2Benchmark put: 10.
	TimesToRun at: #IntMM2Benchmark      put: 10.
	TimesToRun at: #MM2Benchmark         put: 10.
	TimesToRun at: #Perm2Benchmark       put: 10.
	TimesToRun at: #Queens2Benchmark     put: 10.
	TimesToRun at: #Quicksort2Benchmark  put: 10.
	TimesToRun at: #Towers2Benchmark     put: 10.
	TimesToRun at: #TreeSort2Benchmark   put: 10! 
!

BenchmarkRunner initialize!


"*****************************************************************************"
"****************************** Benchmark Classes ****************************"
"*****************************************************************************"

StanfordBenchmarks addSubclass: #AbstractBenchmark
	instanceVariableNames: 'seed '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!AbstractBenchmark someMethodsFor: 'initialization'!

floatBase 
   ^self fixedBase!
fixedBase 
   ^0!	"Added by GVG.  Classes that understand will modify it."

initialize
	seed := 0!

initrand
	seed := 74755!

rand
	seed := seed * 1309 + 13849 bitAnd: 65535.
	^seed!

time
	| ztime tb |
	self print.
	ztime := Time millisecondsToRun: [self run].
	tb := String with: Character tab.
	Transcript show: tb , ztime printString , 'ms;' , tb , (String with: Character cr).
	^ztime! !

"*****************************************************************************"
"******************************** Tak Benchmark ******************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #TakBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TakBenchmark someMethodsFor: 'misc'!

run
	"TakBenchmark new run"
	| r |
	r := self takX: 18 Y: 12 Z: 6.
	r ~= 7 ifTrue: [ self error: 'tak results are incorrect' ].!

takX: x Y: y Z: z 
	y < x ifFalse: [^z]
		ifTrue: [^self
				takX: (self
						takX: x - 1
						Y: y
						Z: z)
				Y: (self
						takX: y - 1
						Y: z
						Z: x)
				Z: (self
						takX: z - 1
						Y: x
						Z: y)]! !


"*****************************************************************************"
"***************************** Quicksort Benchmark ***************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #QuicksortBenchmark
	instanceVariableNames: 'sortlist biggest littlest '
	classVariableNames: 'QSortelements '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!QuicksortBenchmark someMethodsFor: 'implementation'!

initarr
	self initrand.
	biggest := -1000000.
	littlest := 1000000.
	sortlist := Array new: QSortelements. 
	1 to: QSortelements do: 
		[:i | 
		sortlist at: i put: self rand.
		(sortlist at: i)
			> biggest ifTrue: [biggest := sortlist at: i].
		(sortlist at: i)
			< littlest ifTrue: [littlest := sortlist at: i]]!

quicksortA: a L: l R: r 
	" quicksort the array A from start to finish"

	| i j x w |
	i := l.
	j := r.
	x := a at: l + r // 2.
	[i <= j]
		whileTrue: 
			[[(a at: i)
				< x]
				whileTrue: [i := i + 1].
			[x < (a at: j)]
				whileTrue: [j := j - 1].
			i <= j
				ifTrue: 
					[w := a at: i.
					a at: i put: (a at: j).
					a at: j put: w.
					i := i + 1.
					j := j - 1]].
	l < j ifTrue: [self
			quicksortA: a
			L: l
			R: j].
	i < r ifTrue: [self
			quicksortA: a
			L: i
			R: r]! !

!QuicksortBenchmark someMethodsFor: 'run'!

run
	self initarr.
	self
		quicksortA: sortlist
		L: 1
		R: QSortelements.
	((sortlist at: 1)
		~= littlest or: [(sortlist at: QSortelements)
			~= biggest])
		ifTrue: [self error: ' Error in Quick.']! !

!QuicksortBenchmark someMethodsFor: 'constants'!

fixedBase ^1.92! !

!QuicksortBenchmark class someMethodsFor: 'constants'!

initialize
    QSortelements := 5000! !

QuicksortBenchmark initialize!

"*****************************************************************************"

Array addVariableSubclass: #Quicksort2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Quicksort2Array someMethodsFor: 'misc'!

initarr: b

	| biggest littlest |
	biggest := -1000000.
	littlest := 1000000.
	1 to: self size do: [:i |
		self at: i put: b rand.
		(self at: i) > biggest ifTrue: [biggest := self at: i].
		(self at: i) < littlest ifTrue: [littlest := self at: i]
	].
	b biggest: biggest.
	b littlest: littlest.!

quickSort self quickSortFrom: 1 to: self size!

quickSortFrom: l to: r 
	| i j x w |
	i := l.
	j := r.
	x := self at: l + r // 2.
	[i <= j]
		whileTrue: 
			[[(self at: i)
				< x]
				whileTrue: [i := i + 1].
			[x < (self at: j)]
				whileTrue: [j := j - 1].
			i <= j
				ifTrue: 
					[w := self at: i.
					self at: i put: (self at: j).
					self at: j put: w.
					i := i + 1.
					j := j - 1]].
	l < j ifTrue: [self quickSortFrom: l to: j].
	i < r ifTrue: [self quickSortFrom: i to: r]!

sort self quickSort! !

"*****************************************************************************"

QuicksortBenchmark addSubclass: #Quicksort2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Quicksort2Benchmark someMethodsFor: 'misc'!

arrayClass ^Quicksort2Array!

biggest: b biggest := b!

initarr
	self initrand.
	sortlist := self arrayClass new: self n.
	sortlist initarr: self!

littlest: x littlest := x!

n ^QSortelements!

run
	self initarr.
	sortlist sort.
	((sortlist at: 1) ~= littlest or: [(sortlist at: self n ) ~= biggest]) ifTrue: [self error: 'error']
	"Quicksort2Benchmark new run"! !


"*****************************************************************************"
"**************************** BubbleSort Benchmark ***************************"
"*****************************************************************************"

QuicksortBenchmark addSubclass: #BubbleSortBenchmark
	instanceVariableNames: 'top '
	classVariableNames: 'Sortelements SortElements '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BubbleSortBenchmark someMethodsFor: 'run'!

initarr
	self initrand.
	biggest := -1000000.
	littlest := 1000000.
	sortlist := Array new: Sortelements.
	1 to: Sortelements do: 
		[:i | 
		sortlist at: i put: self rand.
		(sortlist at: i)
			> biggest ifTrue: [biggest := sortlist at: i].
		(sortlist at: i)
			< littlest ifTrue: [littlest := sortlist at: i]]!

run   | i  j |
	self initarr.
	top :=    Sortelements.
	[top > 1] whileTrue: [
	    i := 1.
	    [i < top] whileTrue: [
		(sortlist at: i) > (sortlist at: i + 1) ifTrue: [
		    j := sortlist at: i.
		    sortlist at: i put: (sortlist at: i + 1).
		    sortlist at: i + 1 put: j.
		].
		i := i + 1.
	    ].
	    top := top - 1.
	].
	((sortlist at: 1) ~= littlest or: 
	[(sortlist at: Sortelements) ~= biggest]) ifTrue:
	    [self error: 'Error3 in Bubble'  ]! !

!BubbleSortBenchmark someMethodsFor: 'constants'!

fixedBase ^ 1.61! !

!BubbleSortBenchmark class someMethodsFor: 'constants'!

initialize Sortelements := 500 "BubbleSortBenchmark initialize"! !

BubbleSortBenchmark initialize!

"*****************************************************************************"

Quicksort2Array addVariableSubclass: #BubbleSort2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BubbleSort2Array someMethodsFor: 'misc'!

bubbleSort
	| j |
	self size
		to: 1
		by: -1
		do: [:top | 1 to: top - 1 do: [:i | (self at: i)
					> (self at: i + 1)
					ifTrue: 
						[j := self at: i.
						self at: i put: (self at: i + 1).
						self at: i + 1 put: j ]]]!

sort self bubbleSort! !

"*****************************************************************************"

Quicksort2Benchmark addSubclass: #BubbleSort2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BubbleSort2Benchmark someMethodsFor: 'misc'!

arrayClass ^BubbleSort2Array!

n ^500! !

!BubbleSort2Benchmark someMethodsFor: 'constants'!

fixedBase ^ 1.61! !


"*****************************************************************************"
"****************************** Towers Benchmark *****************************"
"*****************************************************************************"

StanfordBenchmarks addSubclass: #TowersBenchmarkElement
	instanceVariableNames: 'discsize next '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TowersBenchmarkElement someMethodsFor: 'access'!

discsize
	^discsize!

discsize: s
	discsize := s!

next
	^next!

next: n
	next := n! !

"*****************************************************************************"

AbstractBenchmark addSubclass: #TowersBenchmark
	instanceVariableNames: 'stack cellspace freelist movesdone '
	classVariableNames: 'Maxcells Stackrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TowersBenchmark someMethodsFor: 'constants'!

fixedBase
	^2.39! !

!TowersBenchmark someMethodsFor: 'run'!

run
	cellspace := Array new: Maxcells.
	1 to: cellspace size do: [:i | cellspace at: i put: TowersBenchmarkElement new].
	stack := Array new: Stackrange. 
	1 to: Maxcells do: [:i | (cellspace at: i)
			next: i - 1].
	freelist := Maxcells.
	self init: 1 to: 14.
	self makenull: 2.
	self makenull: 3.
	movesdone := 0.
	self
		towerI: 1
		j: 2
		k: 14.
	movesdone = 16383 ifFalse: [self error: ' Error in towers.']! !

!TowersBenchmark someMethodsFor: 'implementation'!

error: msg
	super error: 'Error in towers: ', msg!

getelement
	| temp |
	freelist > 0
		ifTrue: 
			[temp := freelist.
			freelist := (cellspace at: freelist) next]
		ifFalse: [self error: 'out of space'].
	^temp!

init: s to: n 
	self makenull: s.
	n
		to: 1
		by: -1
		do: [:discctr | self push: discctr onto: s]!

makenull: s
	stack at: s put: 0!

moveFrom: s1 to: s2 
	self push: (self pop: s1)
		onto: s2.
	movesdone := movesdone + 1!

pop: s 
	| temp1 temp |
	(stack at: s)
		> 0 ifFalse: [^self error: 'nothing to pop'].
	temp1 := (cellspace at: (stack at: s)) discsize.
	temp := (cellspace at: (stack at: s)) next.
	(cellspace at: (stack at: s))
		next: freelist.
	freelist := stack at: s.
	stack at: s put: temp.
	^temp1!

push: i onto: s 
	| localel |
	(stack at: s)
		> 0 ifTrue: [(cellspace at: (stack at: s)) discsize <= i ifTrue: [^self error: 'disc size error']].
	localel := self getelement.
	(cellspace at: localel)
		next: (stack at: s).
	stack at: s put: localel.
	(cellspace at: localel)
		discsize: i!

towerI: i j: j k: k 
	| other |
	k = 1
		ifTrue: [self moveFrom: i to: j]
		ifFalse: 
			[other := 6 - i - j.
			self
				towerI: i
				j: other
				k: k - 1.
			self moveFrom: i to: j.
			self
				towerI: other
				j: j
				k: k - 1]! !


!TowersBenchmark class someMethodsFor: 'constants'!

initialize
    Maxcells := 18. Stackrange := 3 "TowersBenchmark initialize"! !

TowersBenchmark initialize!

"*****************************************************************************"

StanfordBenchmarks addSubclass: #Towers2Disk
	instanceVariableNames: 'discSize next '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Towers2Disk someMethodsFor: 'misc'!

>= d
	^discSize >= d discSize!

discSize
	^discSize!

discSize: s discSize := s!

next
	^next!

next: n next := n! !

"*****************************************************************************"

AbstractBenchmark addSubclass: #Towers2Benchmark
	instanceVariableNames: 'stack movesdone '
	classVariableNames: 'Maxcells Sentinel Stackrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Towers2Benchmark someMethodsFor: 'constants'!

fixedBase
	^2.39! !

!Towers2Benchmark someMethodsFor: 'run'!

makenull: s
	stack at: s put: Sentinel!

run
	stack := Array new: Stackrange.
	self makenull: 1.
	self makenull: 2.
	self makenull: 3.
	self init: 1 to: 14.
	movesdone := 0. 
	self
		towerI: 1
		j: 2
		k: 14.
	movesdone = 16383 ifFalse: [self error: ' Error in towers.'].
	"Towers2Benchmark new run"! !

!Towers2Benchmark someMethodsFor: 'implementation'!

error: msg
	super error: 'Error in towers: ', msg! !

!Towers2Benchmark someMethodsFor: 'misc'!

init: s to: n 

	n
		to: 1
		by: -1
		do: [:discctr | self push: (Towers2Disk new discSize: discctr) onto: s]!

moveFrom: s1 to: s2 
	self push: (self pop: s1)
		onto: s2.
	movesdone := movesdone + 1!

pop: s 
	| result |
	Sentinel == (stack at: s) ifTrue: [self error: 'empty'].
	result := stack at: s.
	stack at: s put: result next.
	^result!

push: d onto: s 
	| localel |
	localel := stack at: s.
	d >= localel ifTrue: [^self error: 'disc size error'].
	stack at: s put: (d next: localel)!

towerI: i j: j k: k 
	| other |
	k = 1
		ifTrue: [self moveFrom: i to: j]
		ifFalse: 
			[other := 6 - i - j.
			self
				towerI: i
				j: other
				k: k - 1.
			self moveFrom: i to: j.
			self
				towerI: other
				j: j
				k: k - 1]! !

!Towers2Benchmark class someMethodsFor: 'constants'!

initialize Maxcells := 18. Stackrange := 3. Sentinel := Towers2Disk new discSize: 15. "Towers2Benchmark initialize"! !

Towers2Benchmark initialize!


"*****************************************************************************"
"******************************* Perm Benchmark ******************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #PermBenchmark
	instanceVariableNames: 'permarray pctr '
	classVariableNames: 'Permcount Permrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!PermBenchmark someMethodsFor: 'running'!

run
	
	pctr := 0.
	Permcount
		timesRepeat: 
			[self initialize.
			self permute: 7].
	pctr ~= (8660 *  Permcount) ifTrue: [self error: 'Error in perm']! !

!PermBenchmark someMethodsFor: 'constants'!

fixedBase 
   ^1.75!

shortPrintString ^'perm'! !

!PermBenchmark someMethodsFor: 'initialization'!

initialize
	permarray := Array new: Permrange.
	1 to: permarray size do: [:i | permarray at: i put: i - 1]! !

!PermBenchmark someMethodsFor: 'implementation'!

permute: n 
	pctr := pctr + 1.
	n ~= 1
		ifTrue: 
			[self permute: n - 1.
			n - 1
				to: 1
				by: -1
				do: 
					[:k | 
					self swap: n with: k.
					self permute: n - 1.
					self swap: n with: k]]!

swap: a with: b 
	| t |
	t := permarray at: a.
	permarray at: a put: (permarray at: b).
	permarray at: b put: t! !

!PermBenchmark class someMethodsFor: 'constants'!

initialize 
    Permcount := 5. Permrange := 10 "PermBenchmark initialize"! !

PermBenchmark initialize!

"*****************************************************************************"

Array addVariableSubclass: #PermArray
	instanceVariableNames: 'pctr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!PermArray someMethodsFor: 'misc'!

pctr 
   ^pctr!

pctr: p pctr := p!

permInitialize
	1 to: self size do: [:i | self at: i put: i - 1]!

permSwap: a with: b
	| t |
	t := self at: a.
	self at: a put: (self at: b).
	self at: b put: t!

permute: n
	pctr := pctr + 1.
	n ~= 1
		ifTrue:
			[self permute: n - 1.
			n - 1
				to: 1
				by: -1
				do:
					[:k |
					self permSwap: n with: k.
					self permute: n - 1.
					self permSwap: n with: k]]! !

"*****************************************************************************"

PermBenchmark addSubclass: #Perm2Benchmark		"GVG Was AbstractBenchmark..."
	instanceVariableNames: ''
	classVariableNames: '' "<---Permcount Permrange deleted by GVG"
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!Perm2Benchmark someMethodsFor: 'misc'!

run | apermarray |
	apermarray :=  PermArray new: Permrange.
	apermarray pctr: 0.
	Permcount timesRepeat: [apermarray permInitialize. apermarray permute: 7].
	apermarray pctr ~= (8660 * Permcount) ifTrue: [ self error: 'error'].
	"Perm2Benchmark new run"! !

!Perm2Benchmark class someMethodsFor: 'constants'!

initialize Permcount := 5. Permrange := 10 "Perm2Benchmark initialize"! !

Perm2Benchmark initialize!


"*****************************************************************************"
"***************************** SumFromTo Benchmark ***************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #SumFromToBenchmark
	instanceVariableNames: 'x '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!SumFromToBenchmark someMethodsFor: 'misc'!

run
	x :=0.
	100 timesRepeat: [self sumFrom: 1 to: 10000]	
	"SumFromToBenchmark new run"!

sumFrom: start to: end
	| sum |
	sum := 0.
	start to: end do: [:i | sum := sum + i].
	^sum! !


"*****************************************************************************"
"**************************** TreeSort Benchmark *****************************"
"*****************************************************************************"

StanfordBenchmarks addSubclass: #TreeSortBenchmarkNode
	instanceVariableNames: 'left right val '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSortBenchmarkNode someMethodsFor: 'access'!

left ^left!

left: l left := l!

right ^right!

right: r right := r!

val ^val!

val: v val := v! !

"*****************************************************************************"

QuicksortBenchmark addSubclass: #TreeSortBenchmark
	instanceVariableNames: 'tree '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSortBenchmark someMethodsFor: 'run'!

run
	self initarr.
	tree := TreeSortBenchmarkNode new val: (sortlist at: 1).
	2 to: QSortelements do: [:i | self insert: (sortlist at: i)
			Into: tree].
	(self checkTree: tree)
		ifFalse: [self error: ' Error in Tree.']! !

!TreeSortBenchmark someMethodsFor: 'implementation'!

checkTree: p 
	| result |
	result := true.
	p left notNil ifTrue: [p left val <= p val
			ifTrue: [result := false]
			ifFalse: [result := (self checkTree: p left)
							and: [result]]].
	p right notNil ifTrue: [p right val >= p val
			ifTrue: [result := false]
			ifFalse: [result := (self checkTree: p right)
							and: [result]]].
	^result!

createNode: n
	^TreeSortBenchmarkNode new val: n!

insert: n Into: t 
	n > t val
		ifTrue: [t left isNil
				ifTrue: [t left: (self createNode: n)]
				ifFalse: [self insert: n Into: t left]]
		ifFalse: [n < t val ifTrue: [t right isNil
					ifTrue: [t right: (self createNode: n)]
					ifFalse: [self insert: n Into: t right]]]! !

!TreeSortBenchmark someMethodsFor: 'constants'!

fixedBase ^2.5! !


"*****************************************************************************"

StanfordBenchmarks addSubclass: #TreeSort2Node
	instanceVariableNames: 'left right val '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSort2Node someMethodsFor: 'misc'!

checkTree
^	(nil == left or: [left val < val and: [left checkTree]]) and:
	[nil == right or: [right val > val and: [right checkTree]]]!

contents: c val := c!

createNode: n
	^self class new contents: n!

insert: n 
	n = val ifTrue: [^self error: 'error'].
	n < val
		ifTrue: [nil == left
				ifTrue: [left := self createNode: n]
				ifFalse: [left insert: n]]
		ifFalse: [nil == right
				ifTrue: [right := self createNode: n]
				ifFalse: [right insert: n]]!

val ^val! !

"*****************************************************************************"

Quicksort2Benchmark addSubclass: #TreeSort2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSort2Benchmark someMethodsFor: 'misc'!

run
	| tree |
	self initarr.
	tree := TreeSort2Node new contents: (sortlist at: 1).
	2 to: self n do: [:i | tree insert: (sortlist at: i)].
	tree checkTree or: [self error: 'error'].
	"TreeSort2Benchmark new run"! !

!TreeSort2Benchmark someMethodsFor: 'constants'!

fixedBase ^2.5! !


"*****************************************************************************"
"****************************** Queens Benchmark *****************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #QueensBenchmark
	instanceVariableNames: ''
	classVariableNames: 'Queencount '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!QueensBenchmark someMethodsFor: 'run'!

run
	Queencount timesRepeat: [self doQueens]! !

!QueensBenchmark someMethodsFor: 'implementation'!

doQueens
	| i a b c x |
	a := Array new: 9.
	b := Array new: 17.
	c := Array new: 15.
	x := Array new: 9.
	i := -7.
	[i <= 16]
		whileTrue: 
			[(i >= 1 and: [i <= 8])
				ifTrue: [a at: i put: true].
			i >= 2 ifTrue: [b at: i put: true].
			i <= 7 ifTrue: [c at: i + 8 put: true].
			i := i + 1].
	(self
		tryI: 1
		A: b
		B: a
		C: c
		X: x)
		ifFalse: [self error: 'Error in queens']!

tryI: i A: a B: b C: c X: x 
	| j q |
	j := 0.
	q := false.
	[q not and: [j ~= 8]]
		whileTrue: 
			[j := j + 1.
			q := false.
			((b at: j)
				and: [(a at: i + j)
						and: [c at: i - j + 8]])
				ifTrue: 
					[x at: i put: j.
					b at: j put: false.
					a at: i + j put: false.
					c at: i - j + 8 put: false.
					i < 8
						ifTrue: 
							[q := self
										tryI: i + 1
										A: a
										B: b
										C: c
										X: x.
							q
								ifFalse: 
									[b at: j put: true.
									a at: i + j put: true.
									c at: i - j + 8 put: true]]
						ifFalse: [q := true]]].
	^q! !

!QueensBenchmark someMethodsFor: 'constants'!

fixedBase ^1.83! !

!QueensBenchmark class someMethodsFor: 'constants'!

initialize Queencount := 50 "QueensBenchmark initialize"! !

QueensBenchmark initialize!

"*****************************************************************************"

AbstractBenchmark addSubclass: #Queens2Benchmark
	instanceVariableNames: 'freeMajs freeRows freeMins qrows '
	classVariableNames: 'Queencount '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Queens2Benchmark someMethodsFor: 'run'!

run
	Queencount timesRepeat: [self initialize doQueens]
	"Queens2Benchmark new run"! !

!Queens2Benchmark someMethodsFor: 'implementation'!

doQueens
	(self try: 1) 
		ifFalse: [self error: 'Error in queens']!

initialize
	freeMajs := (Array new: 15) atAllPut: true.
	freeRows := (Array new: 8) atAllPut: true.
	freeMins := (Array new: 15) atAllPut: true.
	qrows := Array new: 8.!

try: col
	1 to: 8 do: [ :row |
		((freeRows at: row) and: [(freeMajs at: col + row - 1) and: [freeMins at: col - row + 7]]) ifTrue: [
			qrows at: col put: row.
			freeRows at: row put: false.
			freeMajs at: col + row - 1 put: false.
			freeMins at: col - row + 7 put: false.
			8 = col ifTrue: [ ^true ].
			(self try: col + 1 ) ifTrue: [^true].
			freeRows at: row put: true.
			freeMajs at: col + row - 1 put: true.
			freeMins at: col - row + 7 put: true] ].
	^false! !

!Queens2Benchmark someMethodsFor: 'constants'!

fixedBase ^1.83! !

!Queens2Benchmark class someMethodsFor: 'constants'!

initialize Queencount := 50 "Queens2Benchmark initialize"! !

Queens2Benchmark initialize!


"*****************************************************************************"
"******************************* Sieve Benchmark *****************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #SieveBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!SieveBenchmark someMethodsFor: 'misc'!

run
	| size iter flags prime k |
	size := 8190.
	iter := 10.
	flags := (Array new: size + 1)
				atAllPut: true.
	iter timesRepeat: [
		0 to: size do: [:i | (flags at: i + 1) ifTrue: [
			prime := i + i + 3.
			k := i + prime.
			[k <= size] whileTrue: [
				flags at: k put: false.
				k := k + prime]]]]
	"SieveBenchmark new run"! !


"*****************************************************************************"
"****************************** IntMM Benchmark ******************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #IntMMBenchmark
	instanceVariableNames: 'ma mb mr '
	classVariableNames: 'IRowsize '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMMBenchmark someMethodsFor: 'run'!

run
	self initialize.
	mr := self mmMatrix.
	ma := self initmatrix.
	mb := self initmatrix.
	self initrand.
	1 to: IRowsize do: [:i | 1 to: IRowsize do: [:j | (mr at: i)
				at: j put: (self
					innerproductOf: ma
					and: mb
					row: i
					column: j)]]! !

!IntMMBenchmark someMethodsFor: 'implementation'!

initmatrix 
	| m temp |
	m := self mmMatrix.
	1 to: IRowsize do: [:i | 1 to: IRowsize do: 
			[:j | 
			temp := self newValue.
			(m at: i)
				at: j put: temp - (temp // 120 * 120) - 60]].
	^m!

innerproductOf: a and: b row: row column: column 
	| result |
	result := 0.
	1 to: IRowsize do: [:i | result := ((a at: row)
						at: i) * ((b at: i) at: column) + result].
	^result!

makeMatrix: n by: m 
	| r |
	r := Array new: n.
	1 to: r size do: [:i | r at: i put: (Array new: m)].
	^r!

mmMatrix
	^self makeMatrix: IRowsize + 1 by: IRowsize + 1!

newValue
	^self rand! !

!IntMMBenchmark someMethodsFor: 'constants'!

fixedBase ^1.46! !

!IntMMBenchmark class someMethodsFor: 'constants'!

initialize 
    IRowsize := 40 "IntMMBenchmark initialize"! !

IntMMBenchmark initialize!

"*****************************************************************************"

Array addVariableSubclass: #IntMM2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Array someMethodsFor: 'misc'!

innerProductWith: m col: c 
	| sum |
	sum := 0.
	1 to: self size do: [:i | sum := (self at: i)
					* ((m at: i)
							at: c) + sum].
	^sum! !

"*****************************************************************************"

Array addVariableSubclass: #IntMM2Matrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Matrix someMethodsFor: 'misc'!

* m 
	| r |
	r := self class new: self rowSize by: self colSize.
	1 to: self rowSize do: [:i | 1 to: m colSize do: [:j | (r at: i)
				at: j put: ((self at: i)
					innerProductWith: m col: j)]].
	^r!

colSize 
   ^ (self at: 1) size!

init: bench
	| temp |
	1 to: self rowSize do: [ :i |
		1 to: self colSize do: [ :j |
			temp := bench newValue.
			(self at: i) at: j put: temp - ((temp / 120) * 120) - 60]]!

rowSize ^self size! !


!IntMM2Matrix class someMethodsFor: 'misc'!

new: i by: j
	| r |
	r := self new: i.
	1 to: i do: [ :x | r at: x put: (IntMM2Array new: j) ].
	^r! !

IntMMBenchmark addSubclass: #IntMM2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Benchmark someMethodsFor: 'run'!

run
	| m |
	self initrand.
	m := ((IntMM2Matrix new: IRowsize by: IRowsize) init: self) * ((IntMM2Matrix new: IRowsize by: IRowsize) init: self).
	^m
	"IntMM2Benchmark new run"! !


"*****************************************************************************"
"******************************** MM Benchmark *******************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #MMBenchmark
	instanceVariableNames: 'ma mb mr '
	classVariableNames: 'Rowsize '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!MMBenchmark someMethodsFor: 'constants'!

fixedBase ^0.73!

floatBase ^2.92! !

!MMBenchmark someMethodsFor: 'run'!

run
	self initialize.
	mr := self mmMatrix.
	ma := self initmatrix.
	mb := self initmatrix.
	self initrand.
	1 to: Rowsize do: [:i | 1 to: Rowsize do: [:j | (mr at: i)
				at: j put: (self
					innerproductOf: ma
					and: mb
					row: i
					column: j)]]! !

!MMBenchmark someMethodsFor: 'implementation'!

initmatrix 
	| m |
	m := self mmMatrix.
	1 to: Rowsize do: [:i | 
		1 to: Rowsize do: 
			[:j |  (m at: i) at: j put: self newValue]].
	^m!

innerproductOf: a and: b row: row column: column 
	| result |
	result := 0.0.
	1 to: Rowsize do: [:i | result := ((a at: row)
						at: i) * ((b at: i) at: column) + result].
	^result!

makeMatrix: n by: m 
	| r |
	r := Array new: n.
	1 to: r size do: [:i | r at: i put: (Array new: m)].
	^r!

mmMatrix
	^self makeMatrix: Rowsize + 1 by: Rowsize + 1!

newValue
	^self rand asFloat! !

!MMBenchmark class someMethodsFor: 'constants'!

initialize Rowsize := 40 "MMBenchmark initialize"! !

MMBenchmark initialize!

"*****************************************************************************"

Array addVariableSubclass: #MM2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!MM2Array someMethodsFor: 'misc'!

innerProductWith: m col: c 
	| sum |
	sum := 0.0.
	1 to: self size do: [:i | sum := (self at: i)
					* ((m at: i)
							at: c) + sum].
	^sum! !

"*****************************************************************************"

Array addVariableSubclass: #MM2Matrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!MM2Matrix someMethodsFor: 'misc'!

* m 
	| r |
	r := self class new: self rowSize by: self colSize.
	1 to: self rowSize do: [:i | 1 to: m colSize do: [:j | (r at: i)
				at: j put: ((self at: i)
					innerProductWith: m col: j)]].
	^r!

colSize 
   ^ (self at: 1) size!

init: bench
	| temp |
	1 to: self rowSize do: [ :i |
		1 to: self colSize do: [ :j |
			(self at: i) at: j put: bench newValue]]!

rowSize ^self size! !


!MM2Matrix class someMethodsFor: 'misc'!

new: i by: j
	| r |
	r := self new: i.
	1 to: i do: [ :x | r at: x put: (MM2Array new: j) ].
	^r! !

MMBenchmark addSubclass: #MM2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!MM2Benchmark someMethodsFor: 'run'!

run
	| m |
	self initrand.
	m := ((MM2Matrix new: Rowsize by: Rowsize) init: self) * ((MM2Matrix new: Rowsize by: Rowsize) init: self).
	^m
	"MM2Benchmark new run"! !



"*****************************************************************************"
"******************************** FFT Benchmark ******************************"
"*****************************************************************************"

"The FFT benchmark was translated from the C version by Greg Gritton.
 It is mostly a literal translation, implementing all of the logic
 procedurally rather than in the SComplex class.  An actual Smalltalk
 FFT library would probably be implemented differently."

StanfordBenchmarks addSubclass: #SComplex
	instanceVariableNames: 'rp ip'    "Real and Imaginary parts"
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!SComplex someMethodsFor: 'implementation'!

rp
	^rp!

rp: aValue
	rp := aValue!

ip
	^ip!

ip: aValue
	ip := aValue!
    
rp: rVal ip: iVal
	rp := rVal.
	ip := iVal!

* aValue
	^aValue multByComplex: self!

+ aValue
	^aValue addToComplex: self!

- aValue
	^aValue subtractFromComplex: self!

multByComplex: aComplex
	^SComplex newRp: (rp * aComplex rp) - (ip * aComplex ip)
	             ip: (ip * aComplex rp) + (rp * aComplex ip)!

addToComplex: aComplex
	^SComplex newRp: rp + aComplex rp
	             ip: ip + aComplex ip!

subtractFromComplex: aComplex
	^SComplex newRp: aComplex rp - rp
	             ip: aComplex ip - ip!

printOn: aStream
	aStream nextPutAll: self printString!

printString
	^'(', rp printString, '+', ip printString, 'j)'!

!

!SComplex class someMethodsFor: 'instance creation'!

newRp: rp ip: ip
	^super new rp: rp ip: ip!

new
	^self newRp: 0.0 ip: 0.0!
!

!Float someMethodsFor: 'implementation'!
multByComplex: aComplex
	^SComplex newRp: (self * aComplex rp)
	             ip: (self * aComplex rp)!
!

"*****************************************************************************"

AbstractBenchmark addSubclass: #FFTBenchmark
	instanceVariableNames: ''
	classVariableNames: 'FFTSize'
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!FFTBenchmark someMethodsFor: 'implementation'!

exptab: n
	
	"Returns an initialized 'e' complex array"
	"n is FFTSize"
	
	| "float"           theta divisor 
	  "Array(float)"    h 
	  "Array(SComplex)" e
	  "int"             i j k l m |
	  
	theta := 3.1415926536.
	divisor := 4.0.
	h := Array new: 25.
	e := self newComplexArray: (n // 2 + 1).
	 
	1 to: 25 do: [ :index |
		h at: index  put: (1.0/(2.0 * (self cos: (theta/divisor)))).
		divisor := divisor + divisor.
	].
	
	m := n // 2.
	l := m // 2.
	j := 1.
	(e at: 1) rp: 1.0.
	(e at: 1) ip: 0.0.
	(e at: l+1) rp: 0.0.
	(e at: l+1) ip: 1.0.
	(e at: m+1) rp: -1.0.
	(e at: m+1) ip: 0.0.
	
	[
		i := l // 2.
		k := i.
		
		[
			e at: k+1 put: ((e at: k+i+1) + (e at: k-i+1)) * (h at: j).
			k := k + l.
			k <= m   "test"
		] whileTrue: [].
		
		j := j+1 min: 25.
		l := i.
		l > 1  "test"
	] whileTrue: [].
	
	^e
!

FFTn: n z: z w: w e: e sqrinv: sqrinv

	"Compute FFT - Based on C implementation
	 n = number of elements in main arrays
	 z = array1 (modified)
	 w = array2 (modified)
	 e = smaller array
	 sqrinv = some constsant
	 It would be nice if someone would fill out the real meanings of these."
	
	| i j k l m |   "All integers"
	
	m := n // 2.
	l := 1.
	
	[
		k := 0.
		j := l.
		i := 1.
		
		[
			[
				w at: i+k put: (z at: i) + (z at: m+i).
				w at: i+j put: (e at: k+1) * ((z at: i) - (z at: m+i)).
				i := i + 1.
				i <= j   "test"
			] whileTrue: [].
			
			k := j.
			j := k + l.
			j <= m   "test"
		] whileTrue: [].
		
		1 to: n do: [ :index | 
			z at: index put: (w at: index) copy.
		].
		
		l := l+l.     "l:=l*2"
		l <= m  "test"
	] whileTrue: [].
	
	1 to: n do: [ :index |
		(z at: index) rp: sqrinv * (z at: index) rp
		              ip: (0.0 - sqrinv) * (z at: index) ip.
	].
! 

random8K: aSeed
	
	"Return a poor-quality random number in the range of 0-8191"
	
	^ (4855 * aSeed + 1731) bitAnd: 8191
!

newComplexArray: size
	
	"Return an array of SComplex elemtns of a given length"
	
	| a |
	
	a := Array new: size.
	1 to: size do: [ :i | a at: i put: SComplex new ].
	^a
!

cos: x
	
	"Computes cos of x (x in radians) by an expansion"
	
	| "int"   factor
	  "float" result power |
	
	result := 1.0.
	factor := 1.
	power := x.
	
	2 to: 10 do: [ :i |
		factor := factor * i.
		power := power * x.
		((i bitAnd: 1) == 0) ifTrue: [
			((i bitAnd: 3) == 0) 
				ifTrue:  [ result := result + (power / factor) ]
				ifFalse: [ result := result - (power / factor) ].
		].
	].
	^result
!

run
	| z e w |
	
	e := self exptab: FFTSize.
	seed := 5767.
	z := self newComplexArray: FFTSize.
	w := self newComplexArray: FFTSize.
	
	1 to: FFTSize do: [ :i |
		seed := self random8K: seed.
		(z at: i) rp: (20.0 * (seed / 8192.0)) - 10.0.
		seed := self random8K: seed.
		(z at: i) ip: (20.0 * (seed / 8192.0)) - 10.0.
	].
	
	1 to: 20 do: [ :i |
		self FFTn: FFTSize z: z w: w e: e sqrinv: 0.0625.
	].

	((z at: 1) rp < -9.3653 
	 or: [ (z at: 1) rp > -9.3652
	 or: [ (z at: 1) ip < -3.9869 
	 or: [ (z at: 1) ip > -3.9868 ]]]) 
	ifTrue: [ self error: 'Error in FFT' ].
! !

!FFTBenchmark class someMethodsFor: 'constants'!

initialize
	FFTSize := 256
! !

FFTBenchmark initialize!

"*****************************************************************************"
"****************************** Puzzle Benchmark *****************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #PuzzleBenchmark
	instanceVariableNames: 'piececount class piecemax puzzl p n kount '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!PuzzleBenchmark someMethodsFor: 'implementation'!

fitI: i J: j 
	1 to: (piecemax at: i)
		do: [:k | (  ((p at: i) at: k) 
				and: [(puzzl at: j + k) ])
				ifTrue: [^false]].
	^true!

placeI: i J: j 
	1 to: (piecemax at: i)
		do: [:k | ((p at: i) at: k)
				ifTrue: [puzzl at: j + k put: true]].
	piececount at: (class at: i)
		put: (piececount at: (class at: i)) - 1.
	j+1 to: self puzzleSize do: [:k | (puzzl at: k) ifFalse: [^(k - 1)]].
	^0!

removeI: i J: j 
	1 to: (piecemax at: i)
		do: [:k | ((p at: i) at: k)
				ifTrue: [puzzl at: j + k put: false]].
	piececount at: (class at: i)
		put: (piececount at: (class at: i))
				+ 1!

run1
	piececount := Array new: self classmax.
	class := Array new: self typemax.
	piecemax := Array new: self typemax.
	puzzl := Array new: self puzzleSize.
	p := self makePuzzleMatrix: self typemax by: self puzzleSize.
	1 to: self puzzleSize do: [:m | puzzl at: m put: true]!

run2  
	1 to: 5 do: [ :i |
	    1 to: 5 do: [ :j |
	        1 to: 5 do: [ :k |
		    puzzl at: 1 + i + (self d * (j + (self d * k))) put: false ]]].

	1 to: self typemax   do: [ :i |
	    1 to: self puzzleSize   do: [ :m |
		(p at: i) at: m put: false]].

	1 to: 4 do: [ :i | 0 to: 1 do: [ :j | 0 to: 0 do: [ :k | (p at: 1) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 1 put: 1.
	piecemax at: 1 put: 4 + (self d * 1) + (self d * self d * 0).

	1 to: 2 do: [ :i | 0 to: 0 do: [ :j | 0 to: 3 do: [ :k | (p at: 2) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 2 put: 1.
	piecemax at: 2 put: 2 + (self d * 0) + (self d * self d * 3).

	1 to: 1 do: [ :i | 0 to: 3 do: [ :j | 0 to: 1 do: [ :k | (p at: 3) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 3 put: 1.
	piecemax at: 3 put: 1 + (self d * 3) + (self d * self d * 1).

	1 to: 2 do: [ :i | 0 to: 3 do: [ :j | 0 to: 0 do: [ :k | (p at: 4) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 4 put: 1.
	piecemax at: 4 put: 2 + (self d * 3) + (self d * self d * 0).

	1 to: 4 do: [ :i | 0 to: 0 do: [ :j | 0 to: 1 do: [ :k | (p at: 5) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 5 put: 1.
	piecemax at: 5 put: 4 + (self d * 0) + (self d * self d * 1).

	1 to: 1 do: [ :i | 0 to: 1 do: [ :j | 0 to: 3 do: [ :k | (p at: 6) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 6 put: 1.
	piecemax at: 6 put: 1 + (self d * 1) + (self d * self d * 3).

	1 to: 3 do: [ :i | 0 to: 0 do: [ :j | 0 to: 0 do: [ :k | (p at: 7) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 7 put: 2.
	piecemax at: 7 put: 3 + (self d * 0) + (self d * self d * 0).

	1 to: 1 do: [ :i | 0 to: 2 do: [ :j | 0 to: 0 do: [ :k | (p at: 8) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 8 put: 2.
	piecemax at: 8 put: 1 + (self d * 2) + (self d * self d * 0).

	1 to: 1 do: [ :i | 0 to: 0 do: [ :j | 0 to: 2 do: [ :k | (p at: 9) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 9 put: 2.
	piecemax at: 9 put: 1 + (self d * 0) + (self d * self d * 2).

	1 to: 2 do: [ :i | 0 to: 1 do: [ :j | 0 to: 0 do: [ :k | (p at: 10) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 10 put: 3.
	piecemax at: 10 put: 2 + (self d * 1) + (self d * self d * 0).

	1 to: 2 do: [ :i | 0 to: 0 do: [ :j | 0 to: 1 do: [ :k | (p at: 11) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 11 put: 3.
	piecemax at: 11 put: 2 + (self d * 0) + (self d * self d * 1).

	1 to: 1 do: [ :i | 0 to: 1 do: [ :j | 0 to: 1 do: [ :k | (p at: 12) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 12 put: 3.
	piecemax at: 12 put: 1 + (self d * 1) + (self d * self d * 1).

	1 to: 2 do: [ :i | 0 to: 1 do: [ :j | 0 to: 1 do: [ :k | (p at: 13) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 13 put: 4.
	piecemax at: 13 put: 2 + (self d * 1) + (self d * self d * 1).!

trial: j 
	| k |
	kount := kount + 1.
	1 to: self typemax do: [:i | (piececount at: (class at: i))
			~= 0 ifTrue: [(self fitI: i J: j)
				ifTrue: 
					[k := self placeI: i J: j.
					((self trial: k)
						or: [k = 0])
						ifTrue: [^true]
						ifFalse: [self removeI: i J: j]]]].
	^false!

makePuzzleMatrix: nn by: mm 
	| r |
	r := Array new: nn.
	1 to: nn do: [:i | r at: i put: (Array new: mm)].
	^r! 
!

!PuzzleBenchmark someMethodsFor: 'constants'!

classmax ^4!

d ^8!

fixedBase ^0.5!

puzzleSize ^512!

typemax ^13! !

!PuzzleBenchmark someMethodsFor: 'run'!

run
	| m |
	n := 0.
	kount := 0.
	self run1; run2.
	piececount at: 1 put: 13.
	piececount at: 2 put: 3.
	piececount at: 3 put: 1.
	piececount at: 4 put: 1.
	m := 1 + (self d * (1 + (self d * 1))).
	kount := 0.
	(self fitI: 1 J: m) ifTrue: [n := self placeI: 1 J: m] ifFalse: [self error: 'error1 in puzzle'].
	(self trial: n ) ifFalse: [self error: 'error2 in puzzle'].
	kount = 2005 ifFalse: [self error: 'error3 in puzzle']! !


"*****************************************************************************"
"******************************* Takl Benchmark ******************************"
"*****************************************************************************"

StanfordBenchmarks addSubclass: #Cons
	instanceVariableNames: 'car cdr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Cons someMethodsFor: 'comparing'!

= l
	l isNil ifTrue: [ ^ false ] ifFalse: [ ^ car = l car and: [ cdr = l cdr ] ]! !

!Cons someMethodsFor: 'accessing'!

car
	^ car!

car: c
	car := c!

cdr
	^ cdr!

cdr: c
	cdr := c! !

"*****************************************************************************"

AbstractBenchmark addSubclass: #TaklBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TaklBenchmark someMethodsFor: 'misc'!

listn: n
	0 = n ifTrue: [ ^ nil ] ifFalse: [ ^ (Cons new car: n) cdr: (self listn: n - 1) ]!

run
	"TaklBenchmark new run"
	| r |
	r := self taklX: (self listn: 18) Y: (self listn: 12) Z: (self listn: 6).
	(r = (self listn: 7)) ifFalse: [ self error: 'takl results are wrong' ]!

shorterpX: x Y: y
	^ (nil ~= y) and: [ (nil = x) or: [ self shorterpX: x cdr Y: y cdr ] ]!

taklX: x Y: y Z: z
	(self shorterpX: y Y: x) ifFalse: [^z]
		ifTrue: [^self
				taklX: (self
						taklX: x cdr
						Y: y
						Z: z)
				Y: (self
						taklX: y cdr
						Y: z
						Z: x)
				Z: (self
						taklX: z cdr
						Y: x
						Z: y)]! !


"*****************************************************************************"
"*************************** Nested Loop Benchmark ***************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #NestedLoopBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!NestedLoopBenchmark someMethodsFor: 'misc'!

nestedLoopTest
	| sum |
	sum := 0.
	1 to: 100 do: [ :i | 1 to: 100 do: [ :j | sum := sum + 1 ] ].
	^ sum!

run
	"NestedLoopBenchmark new run"
	100 timesRepeat: [ self nestedLoopTest ]! !


"*****************************************************************************"
"***************************** AtAllPut Benchmark ****************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #AtAllPutBenchmark
	instanceVariableNames: ''
	classVariableNames: 'Vect '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!AtAllPutBenchmark someMethodsFor: 'misc'!

run
	"AtAllPutBenchmark new run"
	Vect atAllPut: 7! !

!AtAllPutBenchmark class someMethodsFor: 'initialize'!

initialize
	"AtAllPutBenchmark initialize"
	Vect := Array new: 100000! !

AtAllPutBenchmark initialize!


"*****************************************************************************"
"****************************** SumAll Benchmark *****************************"
"*****************************************************************************"

AtAllPutBenchmark addSubclass: #SumAllBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!SumAllBenchmark someMethodsFor: 'misc'!

run
	| sum |
	sum := 0.
	Vect do: [ :i | sum := sum + 1 ].
	^ sum! !


"*****************************************************************************"
"****************************** Recurse Benchmark ****************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #RecurseBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!RecurseBenchmark someMethodsFor: 'misc'!

run
	"RecurseBenchmark new run"
	10 timesRepeat: [14 recurse]! !


"*****************************************************************************"
"****************************** Sum To Benchmark *****************************"
"*****************************************************************************"

AbstractBenchmark addSubclass: #SumToBenchmark
	instanceVariableNames: 'x '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!SumToBenchmark someMethodsFor: 'misc'!

run
	x := 1.
	100 timesRepeat: [x sumTo: 10000].
	"SumToBenchmark new run"! !

"*****************************************************************************"

AbstractBenchmark addSubclass: #FastSumToBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!FastSumToBenchmark someMethodsFor: 'misc'!

run
	100 timesRepeat: [1 sumTo: 10000]
	"FastSumToBenchmark new run"! !


"*****************************************************************************"
"*************************** IncrementAll Benchmark **************************"
"*****************************************************************************"

AtAllPutBenchmark addSubclass: #IncrementAllBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IncrementAllBenchmark someMethodsFor: 'misc'!

run
	"IncrementAllBenchmark new run"
	1 to: Vect size do: [ :i | Vect at: i put: (Vect at: i) + 1. ]! !





