Object subclass: #Benchmarks
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
!

Benchmarks subclass: #StanfordBenchmarks
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''
!

"*****************************************************************************"
"****************************** Benchmark Runner *****************************"
"*****************************************************************************"

StanfordBenchmarks subclass: #BenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: 'Suite '
	poolDictionaries: ''
	category: 'Self-Benchmarks'
 "library: Environment@#UserLib@#Benchmarks"
!


!BenchmarkRunner methods!

run
	 "BenchmarkRunner new run"
	 | total cTotal fixed cFixed float cFloat size time cTime bm |
	 total _ 0.
	 cTotal _ 0.
	 fixed _ 0.
	 cFixed _ 0.
	 float _ 0.
	 cFloat _ 0.
	 Suite do: 
	 	[:b | 
	 	 bm _ (Smalltalk at: b) new.
	 	 time _ Time millisecondsToRun: [bm run].
	 	 total _ total + time.
	 	 fixed _ fixed + (time * bm fixedBase).
	 	 float _ float + (time * bm floatBase).
	 	 cTime _ bm cTime.
	 	 cTotal _ cTotal + cTime.
	 	 cFixed _ cFixed + (cTime * bm fixedBase).
	 	 cFloat _ cFloat + (cTime * bm floatBase).
	 	 Transcript show: b , ': ' , time printString , ' ms; C time: ' , cTime printString , ' ms; ratio: ' , (time / cTime) asFloat printString.
	 	 Transcript nl].
	 Transcript nl.
	 Transcript show: 'Total: ' , total printString , ' ms; C total: ' , cTotal printString , ' ms; ratio: ' , (total / cTotal) asFloat printString.
	 Transcript nl.
	 size _ Suite size asFloat.
	 fixed _ fixed / size.
	 float _ float / size.
	 cFixed _ cFixed / size.
	 cFloat _ cFloat / size.
	 Transcript show: 'Fixed:  ' , fixed printString , ';  C: ' , cFixed printString , ';  ratio: ' , (fixed / cFixed) asInteger printString.
	 Transcript nl.
	 Transcript show: 'Float:  ' , float printString , ';  C: ' , cFloat printString , ';  ratio: ' , (float / cFloat) asInteger printString.
	 Transcript nl!

runOne
	"BenchmarkRunner initializeNoMM. BenchmarkRunner new runOne"
	| time bm |
	Suite do: 
		[:b | 
		bm _ (Smalltalk at: b) new.
		time _ Time millisecondsToRun: [bm run].
		Transcript show: b , ': ' , time printString , ' ms'.
		Transcript nl].
 	Transcript nl!

runTen
	"BenchmarkRunner new runTen"

	| time bm |
	Suite do: 
		[:b | 
		bm _ (Smalltalk at: b) new.
		time _ Time millisecondsToRun: [10 timesRepeat: [bm run]].
		time _ time / 10.
		Transcript show: b , ': ' , time printString , ' ms'.
		Transcript nl].
	Transcript nl!

runTests
	"BenchmarkRunner initializeNoMM. BenchmarkRunner new runTests"

	| ctime rtime bm |
	Suite do: 
		[:b | 
		bm _ (Smalltalk at: b) new.
		Smalltalk garbageCollect.
		ctime _ Time millisecondsToRun: [10 timesRepeat: [bm run]].
		Smalltalk garbageCollect.
		rtime _ Time millisecondsToRun: [10 timesRepeat: [bm run]].
		ctime _ ctime - rtime.
		rtime _ rtime / 10.
		Transcript show: b , ': ' , ctime printString, ' ms (compile), ', rtime printString , ' ms (run)'.
		Transcript nl].
	Transcript nl! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- Commented out by GVG
BenchmarkRunner class
	instanceVariableNames: ''
 -----"!

!BenchmarkRunner class someMethodsFor: 'initialization'!

add: x
	Suite add: x!

initialize
	Suite _ OrderedCollection new.
	Suite addAll: #(BubbleSortBenchmark BubbleSort2Benchmark 
                 IntMMBenchmark IntMM2Benchmark MMBenchmark MM2Benchmark 
                PermBenchmark Perm2Benchmark QueensBenchmark Queens2Benchmark 
                QuicksortBenchmark Quicksort2Benchmark TowersBenchmark Towers2Benchmark 
                "TreeSortBenchmark  TreeSort2Benchmark" PuzzleBenchmark)
"-----	Suite addAll: #(SieveBenchmark SumToBenchmark FastSumToBenchmark 
                       SumFromToBenchmark RecurseBenchmark AtAllPutBenchmark 
                       SumAllBenchmark IncrementAllBenchmark NestedLoopBenchmark   
                        TakBenchmark TaklBenchmark)
 -----"
	"BenchmarkRunner initialize"!

initializeMM
	Suite _ OrderedCollection new.
	Suite addAll: #( IntMMBenchmark IntMM2Benchmark MMBenchmark MM2Benchmark )
	"BenchmarkRunner initializeMM"!

initializeNoMM
	Suite _ OrderedCollection new.
	Suite addAll: #(BubbleSortBenchmark BubbleSort2Benchmark PermBenchmark Perm2Benchmark QueensBenchmark Queens2Benchmark QuicksortBenchmark Quicksort2Benchmark TowersBenchmark Towers2Benchmark TreeSortBenchmark  TreeSort2Benchmark PuzzleBenchmark SieveBenchmark SumToBenchmark FastSumToBenchmark SumFromToBenchmark RecurseBenchmark AtAllPutBenchmark SumAllBenchmark IncrementAllBenchmark NestedLoopBenchmark TakBenchmark TaklBenchmark)
	"BenchmarkRunner initializeNoMM"! !

BenchmarkRunner initialize!

"*****************************************************************************"
"******************************** Data Classes *******************************"
"*****************************************************************************"

Array variableSubclass: #PermArray
	instanceVariableNames: 'pctr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!PermArray someMethodsFor: 'misc'!

pctr 
   ^pctr!

pctr: p pctr _ p!

permInitialize
	1 to: self size do: [:i | self at: i put: i - 1]!

permSwap: a with: b
	| t |
	t _ self at: a.
	self at: a put: (self at: b).
	self at: b put: t!

permute: n
	pctr _ pctr + 1.
	n ~= 1
		ifTrue:
			[self permute: n - 1.
			n - 1
				to: 1
				by: -1
				do:
					[:k |
					self permSwap: n with: k.
					self permute: n - 1.
					self permSwap: n with: k]]! !

"*****************************************************************************"

Array variableSubclass: #PuzzleArray
        instanceVariableNames: ''
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Self-Benchmarks'!
 
 
!PuzzleArray someMethodsFor: 'accessing'!
 
at: i
        ^super at: i + 1!
 
at: i put: j
        ^super at: i + 1 put: j! !
 
!PuzzleArray someMethodsFor: 'iterating'!
 
do: aBlock 
        "Evaluate aBlock with each of the receiver's elements as the argument."
 
        | index length |
        index _ -1.
        length _ self size - 1.
        [(index _ index + 1) <= length]
                whileTrue: [aBlock value: (self at: index)]! !


"*****************************************************************************"

Array variableSubclass: #Quicksort2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Quicksort2Array someMethodsFor: 'misc'!

initarr: b

	| biggest littlest temp |
	biggest _ -1000000.
	littlest _ 1000000.
	1 to: self size do: [:i |
		temp _ b rand.
		self at: i put: temp - (temp // 100000 * 100000) - 50000.
		(self at: i) > biggest ifTrue: [biggest _ self at: i].
		(self at: i) < littlest ifTrue: [littlest _ self at: i]
	].
	b biggest: biggest.
	b littlest: littlest.!

quickSort self quickSortFrom: 1 to: self size!

quickSortFrom: l to: r 
	| i j x w |
	i _ l.
	j _ r.
	x _ self at: l + r // 2.
	[i <= j]
		whileTrue: 
			[[(self at: i)
				< x]
				whileTrue: [i _ i + 1].
			[x < (self at: j)]
				whileTrue: [j _ j - 1].
			i <= j
				ifTrue: 
					[w _ self at: i.
					self at: i put: (self at: j).
					self at: j put: w.
					i _ i + 1.
					j _ j - 1]].
	l < j ifTrue: [self quickSortFrom: l to: j].
	i < r ifTrue: [self quickSortFrom: i to: r]!

sort self quickSort! !

"*****************************************************************************"

StanfordBenchmarks subclass: #TreeSortNodeBenchmark
	instanceVariableNames: 'left right val '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSortNodeBenchmark someMethodsFor: 'access'!

left ^left!

left: l left _ l!

right ^right!

right: r right _ r!

val ^val!

val: v val _ v! !

"*****************************************************************************"

StanfordBenchmarks subclass: #TowersBenchmarkElement
	instanceVariableNames: 'discsize next '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TowersBenchmarkElement someMethodsFor: 'access'!

discsize
	^discsize!

discsize: s
	discsize _ s!

next
	^next!

next: n
	next _ n! !

"*****************************************************************************"

StanfordBenchmarks subclass: #Cons
	instanceVariableNames: 'car cdr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Cons someMethodsFor: 'comparing'!

= l
	l isNil ifTrue: [ ^ false ] ifFalse: [ ^ car = l car and: [ cdr = l cdr ] ]! !

!Cons someMethodsFor: 'accessing'!

car
	^ car!

car: c
	car _ c!

cdr
	^ cdr!

cdr: c
	cdr _ c! !

"*****************************************************************************"

Quicksort2Array variableSubclass: #BubbleSort2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BubbleSort2Array someMethodsFor: 'misc'!

bubbleSort
	| j |
	self size
		to: 1
		by: -1
		do: [:top | 1 to: top - 1 do: [:i | (self at: i)
					> (self at: i + 1)
					ifTrue: 
						[j _ self at: i.
						self at: i put: (self at: i + 1).
						self at: i + 1 put: j ]]]!

sort self bubbleSort! !

"*****************************************************************************"

Array variableSubclass: #IntMM2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Array someMethodsFor: 'misc'!

innerProductWith: m col: c 
	| sum |
	sum _ 0.
	1 to: self size do: [:i | sum _ (self at: i)
					* ((m at: i)
							at: c) + sum].
	^sum! !

"*****************************************************************************"

Array variableSubclass: #IntMM2Matrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Matrix someMethodsFor: 'misc'!

* m 
	| r |
	r _ self class new: self rowSize by: self colSize.
	1 to: self rowSize do: [:i | 1 to: m colSize do: [:j | (r at: i)
				at: j put: ((self at: i)
					innerProductWith: m col: j)]].
	^r!

colSize 
   ^ (self at: 1) size!

init: bench
	| temp |
	1 to: self rowSize do: [ :i |
		1 to: self colSize do: [ :j |
			temp _ bench newValue.
			(self at: i) at: j put: temp - ((temp / 120) * 120) - 60]]!

rowSize ^self size! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
IntMM2Matrix class
	instanceVariableNames: ''
 -----"!

!IntMM2Matrix class someMethodsFor: 'misc'!

new: i by: j
	| r |
	r _ self new: i.
	1 to: i do: [ :x | r at: x put: (IntMM2Array new: j) ].
	^r! !


"*****************************************************************************"
"****************************** Benchmark Classes ****************************"
"*****************************************************************************"

StanfordBenchmarks subclass: #AbstractBenchmark
	instanceVariableNames: 'seed '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!AbstractBenchmark someMethodsFor: 'initialization'!

cTime
  ^self cTimes: 'macIIcx'!
cTimes: something ^100!	"Added by GVG.  Classes that understand will modify it."

floatBase 
   self fixedBase!
fixedBase 
   ^0!	"Added by GVG.  Classes that understand will modify it."

initialize
	seed _ 0!

initrand
	seed _ 74755!

rand
	seed _ seed * 1309 + 13849 bitAnd: 65535.
	^seed!

time
	| time tb |
	self print.
	time _ Time millisecondsToRun: [self run].
	tb _ String with: Character tab.
	Transcript show: tb , time printString , 'ms;' , tb , 'C time: ' , self cTime printString , ' ms;' , tb , 'ratio: ' , (time / self cTime asFloat "asFloat GVG") printString , (String with: Character cr).
	^time! !

"*****************************************************************************"

AbstractBenchmark subclass: #RecurseBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!RecurseBenchmark someMethodsFor: 'misc'!

run
	"RecurseBenchmark new run"
	10 timesRepeat: [14 recurse]! !

"*****************************************************************************"

StanfordBenchmarks subclass: #Towers2Disk
	instanceVariableNames: 'discSize next '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Towers2Disk someMethodsFor: 'misc'!

>= d
	^discSize >= d discSize!

discSize
	^discSize!

discSize: s discSize _ s!

next
	^next!

next: n next _ n! !

"*****************************************************************************"

StanfordBenchmarks subclass: #TreeSort2Node
	instanceVariableNames: 'left right val '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSort2Node someMethodsFor: 'misc'!

checkTree
^	(nil == left or: [left val < val and: [left checkTree]]) and:
	[nil == right or: [right val > val and: [right checkTree]]]!

contents: c val _ c!

createNode: n
	^self class new contents: n!

insert: n 
	n = val ifTrue: [^self error: 'error'].
	n < val
		ifTrue: [nil == left
				ifTrue: [left _ self createNode: n]
				ifFalse: [left insert: n]]
		ifFalse: [nil == right
				ifTrue: [right _ self createNode: n]
				ifFalse: [right insert: n]]!

val ^val! !

"*****************************************************************************"

AbstractBenchmark subclass: #SumToBenchmark
	instanceVariableNames: 'x '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!SumToBenchmark someMethodsFor: 'misc'!

run
	x _ 1.
	100 timesRepeat: [x sumTo: 10000].
	"SumToBenchmark new run"! !

"*****************************************************************************"

AbstractBenchmark subclass: #FastSumToBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!FastSumToBenchmark someMethodsFor: 'misc'!

run
	100 timesRepeat: [1 sumTo: 10000]
	"FastSumToBenchmark new run"! !

"*****************************************************************************"

AbstractBenchmark subclass: #TakBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TakBenchmark someMethodsFor: 'misc'!

run
	"TakBenchmark new run"
	| r |
	r _ self takX: 18 Y: 12 Z: 6.
	r ~= 7 ifTrue: [ self error: 'tak results are incorrect' ].!

takX: x Y: y Z: z 
	y < x ifFalse: [^z]
		ifTrue: [^self
				takX: (self
						takX: x - 1
						Y: y
						Z: z)
				Y: (self
						takX: y - 1
						Y: z
						Z: x)
				Z: (self
						takX: z - 1
						Y: x
						Z: y)]! !

"*****************************************************************************"

AbstractBenchmark subclass: #QuicksortBenchmark
	instanceVariableNames: 'sortlist biggest littlest '
	classVariableNames: 'QSortelements '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!QuicksortBenchmark someMethodsFor: 'private'!

initarr
	| temp |
	self initrand.
	biggest _ -1000000.
	littlest _ 1000000.
	sortlist _ Array new: QSortelements. 
	1 to: QSortelements do: 
		[:i | 
		temp _ self rand.
		sortlist at: i put: temp - (temp // 100000 * 100000) - 50000.
		(sortlist at: i)
			> biggest ifTrue: [biggest _ sortlist at: i].
		(sortlist at: i)
			< littlest ifTrue: [littlest _ sortlist at: i]]!

quicksortA: a L: l R: r 
	" quicksort the array A from start to finish"

	| i j x w |
	i _ l.
	j _ r.
	x _ a at: l + r // 2.
	[i <= j]
		whileTrue: 
			[[(a at: i)
				< x]
				whileTrue: [i _ i + 1].
			[x < (a at: j)]
				whileTrue: [j _ j - 1].
			i <= j
				ifTrue: 
					[w _ a at: i.
					a at: i put: (a at: j).
					a at: j put: w.
					i _ i + 1.
					j _ j - 1]].
	l < j ifTrue: [self
			quicksortA: a
			L: l
			R: j].
	i < r ifTrue: [self
			quicksortA: a
			L: i
			R: r]! !

!QuicksortBenchmark someMethodsFor: 'run'!

run
	self initarr.
	self
		quicksortA: sortlist
		L: 1
		R: QSortelements.
	((sortlist at: 1)
		~= littlest or: [(sortlist at: QSortelements)
			~= biggest])
		ifTrue: [self error: ' Error in Quick.']! !

!QuicksortBenchmark someMethodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^100].
	mach = 'sun3_160' ifTrue: [^588].
 mach = 'macIIcx'  ifTrue: [^334]!

fixedBase ^1.92! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- Commented out by GVG
QuicksortBenchmark class
	instanceVariableNames: ''
 -----"!

!QuicksortBenchmark class someMethodsFor: 'constants'!

initialize
    QSortelements _ 5000! !

QuicksortBenchmark initialize!


"*****************************************************************************"

QuicksortBenchmark subclass: #Quicksort2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Quicksort2Benchmark someMethodsFor: 'misc'!

arrayClass ^Quicksort2Array!

biggest: b biggest _ b!

initarr
	self initrand.
	sortlist _ self arrayClass new: self n.
	sortlist initarr: self!

littlest: x littlest _ x!

n ^QSortelements!

run
	self initarr.
	sortlist sort.
	((sortlist at: 1) ~= littlest or: [(sortlist at: self n ) ~= biggest]) ifTrue: [self error: 'error']
	"Quicksort2Benchmark new run"! !

"*****************************************************************************"

AbstractBenchmark subclass: #NestedLoopBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!NestedLoopBenchmark someMethodsFor: 'misc'!

nestedLoopTest
	| sum |
	sum _ 0.
	1 to: 100 do: [ :i | 1 to: 100 do: [ :j | sum _ sum + 1 ] ].
	^ sum!

run
	"NestedLoopBenchmark new run"
	100 timesRepeat: [ self nestedLoopTest ]! !

"*****************************************************************************"

AbstractBenchmark subclass: #TowersBenchmark
	instanceVariableNames: 'stack cellspace freelist movesdone '
	classVariableNames: 'Maxcells Stackrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TowersBenchmark someMethodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^190].
	mach = 'sun3_160' ifTrue: [^1130].
 mach = 'macIIcx'  ifTrue: [^1134]!

fixedBase
	^2.39! !

!TowersBenchmark someMethodsFor: 'run'!

run
	cellspace _ Array new: Maxcells.
	1 to: cellspace size do: [:i | cellspace at: i put: TowersBenchmarkElement new].
	stack _ Array new: Stackrange. 
	1 to: Maxcells do: [:i | (cellspace at: i)
			next: i - 1].
	freelist _ Maxcells.
	self init: 1 to: 14.
	self makenull: 2.
	self makenull: 3.
	movesdone _ 0.
	self
		towerI: 1
		j: 2
		k: 14.
	movesdone = 16383 ifFalse: [self error: ' Error in towers.']! !

!TowersBenchmark someMethodsFor: 'private'!

error: msg
	super error: 'Error in towers: ', msg!

getelement
	| temp |
	freelist > 0
		ifTrue: 
			[temp _ freelist.
			freelist _ (cellspace at: freelist) next]
		ifFalse: [self error: 'out of space'].
	^temp!

init: s to: n 
	self makenull: s.
	n
		to: 1
		by: -1
		do: [:discctr | self push: discctr onto: s]!

makenull: s
	stack at: s put: 0!

moveFrom: s1 to: s2 
	self push: (self pop: s1)
		onto: s2.
	movesdone _ movesdone + 1!

pop: s 
	| temp1 temp |
	(stack at: s)
		> 0 ifFalse: [^self error: 'nothing to pop'].
	temp1 _ (cellspace at: (stack at: s)) discsize.
	temp _ (cellspace at: (stack at: s)) next.
	(cellspace at: (stack at: s))
		next: freelist.
	freelist _ stack at: s.
	stack at: s put: temp.
	^temp1!

push: i onto: s 
	| localel |
	(stack at: s)
		> 0 ifTrue: [(cellspace at: (stack at: s)) discsize <= i ifTrue: [^self error: 'disc size error']].
	localel _ self getelement.
	(cellspace at: localel)
		next: (stack at: s).
	stack at: s put: localel.
	(cellspace at: localel)
		discsize: i!

towerI: i j: j k: k 
	| other |
	k = 1
		ifTrue: [self moveFrom: i to: j]
		ifFalse: 
			[other _ 6 - i - j.
			self
				towerI: i
				j: other
				k: k - 1.
			self moveFrom: i to: j.
			self
				towerI: other
				j: j
				k: k - 1]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- Commented out by GVG
TowersBenchmark class
	instanceVariableNames: ''
 -----"!

!TowersBenchmark class someMethodsFor: 'constants'!

initialize
    Maxcells _ 18. Stackrange _ 3 "TowersBenchmark initialize"! !

TowersBenchmark initialize!

"*****************************************************************************"

QuicksortBenchmark subclass: #TreeSortBenchmark
	instanceVariableNames: 'tree '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSortBenchmark someMethodsFor: 'run'!

run
	self initarr.
	tree _ TreeSortNodeBenchmark new val: (sortlist at: 1).
	2 to: QSortelements do: [:i | self insert: (sortlist at: i)
			Into: tree].
	(self checkTree: tree)
		ifFalse: [self error: ' Error in Tree.']! !

!TreeSortBenchmark someMethodsFor: 'private'!

checkTree: p 
	| result |
	result _ true.
	p left notNil ifTrue: [p left val <= p val
			ifTrue: [result _ false]
			ifFalse: [result _ (self checkTree: p left)
							and: [result]]].
	p right notNil ifTrue: [p right val >= p val
			ifTrue: [result _ false]
			ifFalse: [result _ (self checkTree: p right)
							and: [result]]].
	^result!

createNode: n
	^TreeSortNodeBenchmark new val: n!

insert: n Into: t 
	n > t val
		ifTrue: [t left isNil
				ifTrue: [t left: (self createNode: n)]
				ifFalse: [self insert: n Into: t left]]
		ifFalse: [n < t val ifTrue: [t right isNil
					ifTrue: [t right: (self createNode: n)]
					ifFalse: [self insert: n Into: t right]]]! !

!TreeSortBenchmark someMethodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^780].
	mach = 'sun3_160' ifTrue: [^1160].
 mach = 'macIIcx'  ifTrue: [^1183]!

fixedBase ^2.5! !

"*****************************************************************************"

AbstractBenchmark subclass: #PermBenchmark
	instanceVariableNames: 'permarray pctr '
	classVariableNames: 'Permcount Permrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!PermBenchmark someMethodsFor: 'running'!

run
	
	pctr _ 0.
	Permcount
		timesRepeat: 
			[self initialize.
			self permute: 7].
	pctr ~= (8660 *  Permcount) ifTrue: [self error: 'Error in perm']! !

!PermBenchmark someMethodsFor: 'constants'!

cTimes: mach
 mach = 'sun4_260' ifTrue: [^110].
 mach = 'sun3_160' ifTrue: [^748].
 mach = 'macIIcx'  ifTrue: [^1017]!

fixedBase 
   ^1.75!

shortPrintString ^'perm'! !

!PermBenchmark someMethodsFor: 'initialization'!

initialize
	permarray _ Array new: Permrange.
	1 to: permarray size do: [:i | permarray at: i put: i - 1]! !

!PermBenchmark someMethodsFor: 'private'!

permute: n 
	pctr _ pctr + 1.
	n ~= 1
		ifTrue: 
			[self permute: n - 1.
			n - 1
				to: 1
				by: -1
				do: 
					[:k | 
					self swap: n with: k.
					self permute: n - 1.
					self swap: n with: k]]!

swap: a with: b 
	| t |
	t _ permarray at: a.
	permarray at: a put: (permarray at: b).
	permarray at: b put: t! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
PermBenchmark class
	instanceVariableNames: ''
 -----"!

!PermBenchmark class someMethodsFor: 'constants'!

initialize 
    Permcount _ 5. Permrange _ 10 "PermBenchmark initialize"! !

PermBenchmark initialize!


"*****************************************************************************"

PermBenchmark subclass: #Perm2Benchmark		"GVG Was AbstractBenchmark..."
	instanceVariableNames: ''
	classVariableNames: '' "<---Permcount Permrange deleted by GVG"
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Perm2Benchmark someMethodsFor: 'misc'!

run | permarray |
	permarray _  PermArray new: Permrange.
	permarray pctr: 0.
	Permcount timesRepeat: [permarray permInitialize. permarray permute: 7].
	permarray pctr ~= (8660 * Permcount) ifTrue: [ self error: 'error'].
	"Perm2Benchmark new run"! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
Perm2Benchmark class
	instanceVariableNames: ''
 -----"!

!Perm2Benchmark class someMethodsFor: 'constants'!

initialize Permcount _ 5. Permrange _ 10 "Perm2Benchmark initialize"! !

Perm2Benchmark initialize!

"*****************************************************************************"

AbstractBenchmark subclass: #SumFromToBenchmark
	instanceVariableNames: 'x '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!SumFromToBenchmark someMethodsFor: 'misc'!

run
	x _0.
	100 timesRepeat: [self sumFrom: 1 to: 10000]	
	"SumFromToBenchmark new run"!

sumFrom: start to: end
	| sum |
	sum _ 0.
	start to: end do: [:i | sum _ sum + i].
	^sum! !

"*****************************************************************************"

AbstractBenchmark subclass: #IntMMBenchmark
	instanceVariableNames: 'ma mb mr '
	classVariableNames: 'IRowsize '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMMBenchmark someMethodsFor: 'run'!

run
	self initialize.
	mr _ self mmMatrix.
	ma _ self initmatrix.
	mb _ self initmatrix.
	self initrand.
	1 to: IRowsize do: [:i | 1 to: IRowsize do: [:j | (mr at: i)
				at: j put: (self
					innerproductOf: ma
					and: mb
					row: i
					column: j)]]! !

!IntMMBenchmark someMethodsFor: 'private'!

initmatrix 
	| m temp |
	m _ self mmMatrix.
	1 to: IRowsize do: [:i | 1 to: IRowsize do: 
			[:j | 
			temp _ self newValue.
			(m at: i)
				at: j put: temp - (temp // 120 * 120) - 60]].
	^m!

innerproductOf: a and: b row: row column: column 
	| result |
	result _ 0.
	1 to: IRowsize do: [:i | result _ ((a at: row)
						at: i) * ((b at: i) at: column) + result].
	^result!

makeMatrix: n by: m 
	| r |
	r _ Array new: n.
	1 to: r size do: [:i | r at: i put: (Array new: m)].
	^r!

makePuzzleMatrix: n by: m 
	| r |
	r _ PuzzleArray new: n.
	0 to: r size - 1 do: [:i | r at: i put: (PuzzleArray new: m)].
	^r!

mmMatrix
	^self makeMatrix: IRowsize + 1 by: IRowsize + 1!

newValue
	^self rand! !

!IntMMBenchmark someMethodsFor: 'constants'!

cTimes: mach
 mach = 'sun4_260' ifTrue: [^270].
 mach = 'sun3_160' ifTrue: [^928].
 mach = 'macIIcx'  ifTrue: [^400]!

fixedBase ^1.46! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
IntMMBenchmark class
	instanceVariableNames: ''
 -----"!

!IntMMBenchmark class someMethodsFor: 'constants'!

initialize 
    IRowsize _ 40 "IntMMBenchmark initialize"! !

IntMMBenchmark initialize!

