# ----------------------------- bench.py ----------------------------------

# Python version of Stanford Integer Benchmarks

import time
import string

# -------------------------- Module Variables ------------------------------

#Constants
true = 1
false = 0

#Variables
seed = 74755

# ------------------------- Utility Functions ------------------------------

def Initrand():
	#Initialize seed for random number generator
	global seed
	seed = 74755

def Rand():
	#A very poor random number generator
	global seed
	seed = (seed * 1309 + 13849) & 65535
	return seed

# ---------------------------- Bubble Sort ---------------------------------

bSortElements = 500 

def Bubble():
    # "Bubble sort benchmark - creates and sorts an array of size bSortElements."
	size = bSortElements
	sortlist = InitSortlist(size)
	top = size - 1
	while top > 1:
		i = 0
		while i < top:
			if sortlist[i] > sortlist[i+1]:
				j = sortlist[i]
				sortlist[i] = sortlist[i+1]
				sortlist[i+1] = j
			i = i + 1
		top = top - 1
	if sortlist[0] != min(sortlist) or sortlist[size-1] != max(sortlist):
		print "Error in Quicksort"

def InitSortlist(size):
	sortlist = [None]*size;
	for i in range(size):
		temp = Rand()
		sortlist[i] = temp - (temp/100000)*100000 - 50000;
	return sortlist


# ---------------------------- Quicksort ---------------------------------

qSortElements = 5000

def Quick():
	size = qSortElements
	sortlist = InitSortlist(size)
	Quicksort(sortlist, 0, size-1)
	if sortlist[0] != min(sortlist) or sortlist[size-1] != max(sortlist):
		print "Error in Quicksort"

def Quicksort(a, l, r):
	i = l
	j = r
	x = a[(l+r)/2]
	if i > j:
		raise "I didn't expect this"
	while i <= j:
		while a[i] < x: i=i+1
		while x < a[j]: j=j-1
		if i <= j:
			w = a[i]
			a[i] = a[j]
			a[j] = w
			i = i+1
			j = j-1
	if l < j: Quicksort(a, l, j)
	if i < r: Quicksort(a, i, r)

# InitSortlist is in Bubble Sort


# ---------------------------- Fast Sort ---------------------------------

# Fast sort uses languages built-in sorting function

def Fast():
	size = qSortElements
	sortlist = InitFastSortlist(size)
	FastSort(sortlist)
	if sortlist[0] != min(sortlist) or sortlist[size-1] != max(sortlist):
		print "Error in Fastsort"

def FastSort(sortlist):
	sortlist.sort()
	
# InitSortlist is in Bubble Sort


# -------------------- Permutation (using globals) -------------------------

# Permutation program, heavily recursive, written by Denny Brown.

pctr = 0
permarray = [0]

def PermG():
	global pctr
	pctr = 0
	for i in range(5):
		InitPermG()
		PermuteG(7)
	if pctr != 43300:
		print "Error in Perm."

def PermuteG(n):
	global pctr, permarray
	pctr = pctr + 1
	if n != 1:
		PermuteG(n-1)
		k = n - 1
		while k >= 1:
			SwapG(permarray, n, k)
			PermuteG(n - 1)
			SwapG(permarray, n, k)
			k = k - 1

def SwapG(list, index1, index2):
	t = list[index1]
	list[index1] = list[index2]
	list[index2] = t

def InitPermG():
	global permarray
	permarray = [0]            #C code starts at index 1
	for i in range(7):
		permarray.append(i)
	

# --------------------- Permutation (using locals) -------------------------

# Permutation program, heavily recursive, written by Denny Brown.

pctr = 0

def PermL():
	global pctr
	pctr = 0
	for i in range(5):
		permarray = InitPermL()
		PermuteL(permarray, 7)
	if pctr != 43300:
		print "Error in Perm."

def PermuteL(permarray, n):
	global pctr
	pctr = pctr + 1
	if n != 1:
		PermuteL(permarray, n-1)
		k = n - 1
		while k >= 1:
			Swap(permarray, n, k)
			PermuteL(permarray, n - 1)
			Swap(permarray, n, k)
			k = k - 1

def Swap(list, index1, index2):
	t = list[index1]
	list[index1] = list[index2]
	list[index2] = t

def InitPermL():
	permarray = [0]            #C code starts at index 1
	for i in range(7):
		permarray.append(i)
	return permarray





# ------------------------- Towers of Hanoi --------------------------------

#Constants
maxcells = 18
stackrange = 3

#Global variables
stack = []
freelist = 0
movesdone = 0

class Element:
	next = 0
	discsize = 0

cellspace = [None]    #Will contain elements

#Functions

def InitCellspace():
	global cellspace, maxcells
	cellspace = [None]
	for i in range(maxcells):
		cellspace.append(Element())

def InitStack():
	for i in range(stackrange + 1):
		stack.append(0)

def Error(s):
	print "Towers error: ", s
	raise s

def Makenull(s):
	global stack
	stack[s] = 0

def Getelement():
	global freelist, cellspace
	temp = freelist
	if freelist > 0:
		freelist = cellspace[freelist].next
	else:
		Error("out of space   ")
	return temp

def Push(i, s):
	global cellspace, stack
	errorfound = false
	if stack[s] > 0:
		if cellspace[stack[s]].discsize <= i:
			errorfound = true
			Error("disc size error")
	if not errorfound:
		localel = Getelement()
		cellspace[localel].next = stack[s]
		stack[s] = localel
		cellspace[localel].discsize = i

def Init(s, n):
	Makenull(s)
	discctr = n
	while discctr >= 1:
		Push(discctr, s)
		discctr = discctr - 1

def Pop(s):
	global cellspace, freelist, stack
	if (stack[s] > 0):
		temp1 = cellspace[stack[s]].discsize
		temp = cellspace[stack[s]].next
		cellspace[stack[s]].next = freelist
		freelist = stack[s]
		stack[s] = temp
		return temp1
	else:
		Error("nothing to pop.")

def Move(s1, s2):
	global movesdone
	Push(Pop(s1), s2)
	movesdone = movesdone + 1

def tower(i, j, k):
	if k == 1:
		Move(i, j)
	else:
		other = 6 - i - j
		tower(i, other, k-1)
		Move(i, j)
		tower(other, j, k-1)

def Towers():
	global cellspace, freelist, maxcells, movesdone
	InitStack()
	InitCellspace()
	for i in range(maxcells):
		cellspace[i+1].next = i
	freelist = maxcells
	Init(1, 14)
	Makenull(2)
	Makenull(3)
	movesdone = 0
	tower(1, 2, 14)
	if movesdone != 16383:
		print "Error in Towers."


# -------------------------- Run Benchmarks ------------------------------

def runBench(bench, name):
	starttime = time.clock()
	bench()
	print string.ljust(name, 16) + ":", int((time.clock() - starttime) * 1000), "ms"

def go():
	runBench(Bubble, "Bubble")
	runBench(Quick,  "Quick" )
	runBench(Fast,   "Fast"  )
	runBench(PermG,  "Perm (globals)")
	runBench(PermL,  "Perm (locals)" )
	runBench(Towers, "Towers")


