"*****************************************************************************"
"****************************** Benchmark Runner *****************************"
"*****************************************************************************"

Object subclass: #BenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: 'Suite '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BenchmarkRunner methodsFor: 'running'!

run
	"BenchmarkRunner new run"
	| total cTotal fixed cFixed float cFloat size time cTime bm |
	total _ 0.
	cTotal _ 0.
	fixed _ 0.
	cFixed _ 0.
	float _ 0.
	cFloat _ 0.
	Suite do: 
		[:b | 
		bm _ (Smalltalk at: b) new.
		time _ Time millisecondsToRun: [bm run].
		total _ total + time.
		fixed _ fixed + (time * bm fixedBase).
		float _ float + (time * bm floatBase).
		cTime _ bm cTime.
		cTotal _ cTotal + cTime.
		cFixed _ cFixed + (cTime * bm fixedBase).
		cFloat _ cFloat + (cTime * bm floatBase).
		Transcript show: b , ': ' , time printString , ' ms; C time: ' , cTime printString , ' ms; ratio: ' , (time / cTime) asFloat printString.
		Transcript nl].
	Transcript nl.
	Transcript show: 'Total: ' , total printString , ' ms; C total: ' , cTotal printString , ' ms; ratio: ' , (total / cTotal) asFloat printString.
	Transcript nl.
	size _ Suite size asFloat.
	fixed _ fixed / size.
	float _ float / size.
	cFixed _ cFixed / size.
	cFloat _ cFloat / size.
	Transcript show: 'Fixed:  ' , fixed printString , ';  C: ' , cFixed printString , ';  ratio: ' , (fixed / cFixed) asInteger printString.
	Transcript nl.
	Transcript show: 'Float:  ' , float printString , ';  C: ' , cFloat printString , ';  ratio: ' , (float / cFloat) asInteger printString.
	Transcript nl!

runOne
	"BenchmarkRunner initializeNoMM. BenchmarkRunner new runOne"
	| time bm |
	Suite do: 
		[:b | 
		bm _ (Smalltalk at: b) new.
		time _ Time millisecondsToRun: [bm run].
		Transcript show: b , ': ' , time printString , ' ms'.
		Transcript nl].
	Transcript nl!

runTen
	"BenchmarkRunner new runTen"

	| time bm |
	Suite do: 
		[:b | 
		bm _ (Smalltalk at: b) new.
		time _ Time millisecondsToRun: [10 timesRepeat: [bm run]].
		time _ time / 10.
		Transcript show: b , ': ' , time printString , ' ms'.
		Transcript nl].
	Transcript nl!

runTests
	"BenchmarkRunner initializeNoMM. BenchmarkRunner new runTests"

	| ctime rtime bm |
	Suite do: 
		[:b | 
		bm _ (Smalltalk at: b) new.
		Smalltalk garbageCollect.
		ctime _ Time millisecondsToRun: [10 timesRepeat: [bm run]].
		Smalltalk garbageCollect.
		rtime _ Time millisecondsToRun: [10 timesRepeat: [bm run]].
		ctime _ ctime - rtime.
		rtime _ rtime / 10.
		Transcript show: b , ': ' , ctime printString, ' ms (compile), ', rtime printString , ' ms (run)'.
		Transcript nl].
	Transcript nl! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- Commented out by GVG
BenchmarkRunner class
	instanceVariableNames: ''
 -----"!

!BenchmarkRunner class methodsFor: 'initialization'!

add: x
	Suite add: x!

initialize
	Suite _ OrderedCollection new.
	Suite addAll: #(BubbleSortBenchmark BubbleSort2Benchmark IntMMBenchmark IntMM2Benchmark MMBenchmark MM2Benchmark PermBenchmark Perm2Benchmark QueensBenchmark Queens2Benchmark QuicksortBenchmark Quicksort2Benchmark TowersBenchmark Towers2Benchmark TreeSortBenchmark  TreeSort2Benchmark PuzzleBenchmark)
"-----	Suite addAll: #(SieveBenchmark SumToBenchmark FastSumToBenchmark SumFromToBenchmark RecurseBenchmark AtAllPutBenchmark SumAllBenchmark IncrementAllBenchmark NestedLoopBenchmark TakBenchmark TaklBenchmark)
 -----"
	"BenchmarkRunner initialize"!

initializeMM
	Suite _ OrderedCollection new.
	Suite addAll: #( IntMMBenchmark IntMM2Benchmark MMBenchmark MM2Benchmark )
	"BenchmarkRunner initializeMM"!

initializeNoMM
	Suite _ OrderedCollection new.
	Suite addAll: #(BubbleSortBenchmark BubbleSort2Benchmark PermBenchmark Perm2Benchmark QueensBenchmark Queens2Benchmark QuicksortBenchmark Quicksort2Benchmark TowersBenchmark Towers2Benchmark TreeSortBenchmark  TreeSort2Benchmark PuzzleBenchmark SieveBenchmark SumToBenchmark FastSumToBenchmark SumFromToBenchmark RecurseBenchmark AtAllPutBenchmark SumAllBenchmark IncrementAllBenchmark NestedLoopBenchmark TakBenchmark TaklBenchmark)
	"BenchmarkRunner initializeNoMM"! !

BenchmarkRunner initialize!

"*****************************************************************************"
"******************************** Data Classes *******************************"
"*****************************************************************************"

Array variableSubclass: #PermArray
	instanceVariableNames: 'pctr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!PermArray methodsFor: 'misc'!

pctr ^ pctr!

pctr: p pctr _ p!

permInitialize
	1 to: self size do: [:i | self at: i put: i - 1]!

permSwap: a with: b
	| t |
	t _ self at: a.
	self at: a put: (self at: b).
	self at: b put: t!

permute: n
	pctr _ pctr + 1.
	n ~= 1
		ifTrue:
			[self permute: n - 1.
			n - 1
				to: 1
				by: -1
				do:
					[:k |
					self permSwap: n with: k.
					self permute: n - 1.
					self permSwap: n with: k]]! !

"*****************************************************************************"

Array variableSubclass: #PuzzleArray
        instanceVariableNames: ''
        classVariableNames: ''
        poolDictionaries: ''
        category: 'Self-Benchmarks'!
 
 
!PuzzleArray methodsFor: 'accessing'!
 
at: i
        ^super at: i + 1!
 
at: i put: j
        ^super at: i + 1 put: j! !
 
!PuzzleArray methodsFor: 'iterating'!
 
do: aBlock 
        "Evaluate aBlock with each of the receiver's elements as the argument."
 
        | index length |
        index _ -1.
        length _ self size - 1.
        [(index _ index + 1) <= length]
                whileTrue: [aBlock value: (self at: index)]! !


"*****************************************************************************"

Array variableSubclass: #Quicksort2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Quicksort2Array methodsFor: 'misc'!

initarr: b

	| biggest littlest temp |
	biggest _ -1000000.
	littlest _ 1000000.
	1 to: self size do: [:i |
		temp _ b rand.
		self at: i put: temp - (temp // 100000 * 100000) - 50000.
		(self at: i) > biggest ifTrue: [biggest _ self at: i].
		(self at: i) < littlest ifTrue: [littlest _ self at: i]
	].
	b biggest: biggest.
	b littlest: littlest.!

quickSort self quickSortFrom: 1 to: self size!

quickSortFrom: l to: r 
	| i j x w |
	i _ l.
	j _ r.
	x _ self at: l + r // 2.
	[i <= j]
		whileTrue: 
			[[(self at: i)
				< x]
				whileTrue: [i _ i + 1].
			[x < (self at: j)]
				whileTrue: [j _ j - 1].
			i <= j
				ifTrue: 
					[w _ self at: i.
					self at: i put: (self at: j).
					self at: j put: w.
					i _ i + 1.
					j _ j - 1]].
	l < j ifTrue: [self quickSortFrom: l to: j].
	i < r ifTrue: [self quickSortFrom: i to: r]!

sort self quickSort! !

"*****************************************************************************"

Object subclass: #TreeSortNodeBenchmark
	instanceVariableNames: 'left right val '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSortNodeBenchmark methodsFor: 'access'!

left ^left!

left: l left _ l!

right ^right!

right: r right _ r!

val ^val!

val: v val _ v! !

"*****************************************************************************"

Object subclass: #TowersBenchmarkElement
	instanceVariableNames: 'discsize next '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TowersBenchmarkElement methodsFor: 'access'!

discsize
	^discsize!

discsize: s
	discsize _ s!

next
	^next!

next: n
	next _ n! !

"*****************************************************************************"

Object subclass: #Cons
	instanceVariableNames: 'car cdr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Cons methodsFor: 'comparing'!

= l
	l isNil ifTrue: [ ^ false ] ifFalse: [ ^ car = l car and: [ cdr = l cdr ] ]! !

!Cons methodsFor: 'accessing'!

car
	^ car!

car: c
	car _ c!

cdr
	^ cdr!

cdr: c
	cdr _ c! !

"*****************************************************************************"

Quicksort2Array variableSubclass: #BubbleSort2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BubbleSort2Array methodsFor: 'misc'!

bubbleSort
	| j |
	self size
		to: 1
		by: -1
		do: [:top | 1 to: top - 1 do: [:i | (self at: i)
					> (self at: i + 1)
					ifTrue: 
						[j _ self at: i.
						self at: i put: (self at: i + 1).
						self at: i + 1 put: j ]]]!

sort self bubbleSort! !

"*****************************************************************************"

Array variableSubclass: #IntMM2Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Array methodsFor: 'misc'!

innerProductWith: m col: c 
	| sum |
	sum _ 0.
	1 to: self size do: [:i | sum _ (self at: i)
					* ((m at: i)
							at: c) + sum].
	^sum! !

"*****************************************************************************"

Array variableSubclass: #IntMM2Matrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Matrix methodsFor: 'misc'!

* m 
	| r |
	r _ self class new: self rowSize by: self colSize.
	1 to: self rowSize do: [:i | 1 to: m colSize do: [:j | (r at: i)
				at: j put: ((self at: i)
					innerProductWith: m col: j)]].
	^r!

colSize ^ (self at: 1) size!

init: bench
	| temp |
	1 to: self rowSize do: [ :i |
		1 to: self colSize do: [ :j |
			temp _ bench newValue.
			(self at: i) at: j put: temp - ((temp / 120) * 120) - 60]]!

rowSize ^self size! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
IntMM2Matrix class
	instanceVariableNames: ''
 -----"!

!IntMM2Matrix class methodsFor: 'misc'!

new: i by: j
	| r |
	r _ self new: i.
	1 to: i do: [ :x | r at: x put: (IntMM2Array new: j) ].
	^r! !


"*****************************************************************************"
"****************************** Benchmark Classes ****************************"
"*****************************************************************************"

Object subclass: #AbstractBenchmark
	instanceVariableNames: 'seed '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!AbstractBenchmark methodsFor: 'initialization'!

cTime
  ^self cTimes: 'macIIcx'!
cTimes: something ^100!	"Added by GVG.  Classes that understand will modify it."

floatBase ^ self fixedBase!
fixedBase ^0!	"Added by GVG.  Classes that understand will modify it."

initialize
	seed _ 0!

initrand
	seed _ 74755!

rand
	seed _ seed * 1309 + 13849 bitAnd: 65535.
	^seed!

time
	| time tb |
	self print.
	time _ Time millisecondsToRun: [self run].
	tb _ String with: Character tab.
	Transcript show: tb , time printString , 'ms;' , tb , 'C time: ' , self cTime printString , ' ms;' , tb , 'ratio: ' , (time / self cTime asFloat "asFloat GVG") printString , (String with: Character cr).
	^time! !

"*****************************************************************************"

AbstractBenchmark subclass: #RecurseBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!RecurseBenchmark methodsFor: 'misc'!

run
	"RecurseBenchmark new run"
	10 timesRepeat: [14 recurse]! !

"*****************************************************************************"

Object subclass: #Towers2Disk
	instanceVariableNames: 'discSize next '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Towers2Disk methodsFor: 'misc'!

>= d
	^discSize >= d discSize!

discSize
	^discSize!

discSize: s discSize _ s!

next
	^next!

next: n next _ n! !

"*****************************************************************************"

Object subclass: #TreeSort2Node
	instanceVariableNames: 'left right val '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSort2Node methodsFor: 'misc'!

checkTree
^	(nil == left or: [left val < val and: [left checkTree]]) and:
	[nil == right or: [right val > val and: [right checkTree]]]!

contents: c val _ c!

createNode: n
	^self class new contents: n!

insert: n 
	n = val ifTrue: [^self error: 'error'].
	n < val
		ifTrue: [nil == left
				ifTrue: [left _ self createNode: n]
				ifFalse: [left insert: n]]
		ifFalse: [nil == right
				ifTrue: [right _ self createNode: n]
				ifFalse: [right insert: n]]!

val ^val! !

"*****************************************************************************"

AbstractBenchmark subclass: #SumToBenchmark
	instanceVariableNames: 'x '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!SumToBenchmark methodsFor: 'misc'!

run
	x _ 1.
	100 timesRepeat: [x sumTo: 10000].
	"SumToBenchmark new run"! !

"*****************************************************************************"

AbstractBenchmark subclass: #FastSumToBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!FastSumToBenchmark methodsFor: 'misc'!

run
	100 timesRepeat: [1 sumTo: 10000]
	"FastSumToBenchmark new run"! !

"*****************************************************************************"

AbstractBenchmark subclass: #TakBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TakBenchmark methodsFor: 'misc'!

run
	"TakBenchmark new run"
	| r |
	r _ self takX: 18 Y: 12 Z: 6.
	r ~= 7 ifTrue: [ self error: 'tak results are incorrect' ].!

takX: x Y: y Z: z 
	y < x ifFalse: [^z]
		ifTrue: [^self
				takX: (self
						takX: x - 1
						Y: y
						Z: z)
				Y: (self
						takX: y - 1
						Y: z
						Z: x)
				Z: (self
						takX: z - 1
						Y: x
						Z: y)]! !

"*****************************************************************************"

AbstractBenchmark subclass: #QuicksortBenchmark
	instanceVariableNames: 'sortlist biggest littlest '
	classVariableNames: 'QSortelements '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!QuicksortBenchmark methodsFor: 'private'!

initarr
	| temp |
	self initrand.
	biggest _ -1000000.
	littlest _ 1000000.
	sortlist _ Array new: QSortelements. 
	1 to: QSortelements do: 
		[:i | 
		temp _ self rand.
		sortlist at: i put: temp - (temp // 100000 * 100000) - 50000.
		(sortlist at: i)
			> biggest ifTrue: [biggest _ sortlist at: i].
		(sortlist at: i)
			< littlest ifTrue: [littlest _ sortlist at: i]]!

quicksortA: a L: l R: r 
	" quicksort the array A from start to finish"

	| i j x w |
	i _ l.
	j _ r.
	x _ a at: l + r // 2.
	[i <= j]
		whileTrue: 
			[[(a at: i)
				< x]
				whileTrue: [i _ i + 1].
			[x < (a at: j)]
				whileTrue: [j _ j - 1].
			i <= j
				ifTrue: 
					[w _ a at: i.
					a at: i put: (a at: j).
					a at: j put: w.
					i _ i + 1.
					j _ j - 1]].
	l < j ifTrue: [self
			quicksortA: a
			L: l
			R: j].
	i < r ifTrue: [self
			quicksortA: a
			L: i
			R: r]! !

!QuicksortBenchmark methodsFor: 'run'!

run
	self initarr.
	self
		quicksortA: sortlist
		L: 1
		R: QSortelements.
	((sortlist at: 1)
		~= littlest or: [(sortlist at: QSortelements)
			~= biggest])
		ifTrue: [self error: ' Error in Quick.']! !

!QuicksortBenchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^100].
	mach = 'sun3_160' ifTrue: [^588].
 mach = 'macIIcx'  ifTrue: [^334]!

fixedBase ^1.92! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- Commented out by GVG
QuicksortBenchmark class
	instanceVariableNames: ''
 -----"!

!QuicksortBenchmark class methodsFor: 'constants'!

initialize
    QSortelements _ 5000! !

QuicksortBenchmark initialize!


"*****************************************************************************"

QuicksortBenchmark subclass: #Quicksort2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Quicksort2Benchmark methodsFor: 'misc'!

arrayClass ^Quicksort2Array!

biggest: b biggest _ b!

initarr
	self initrand.
	sortlist _ self arrayClass new: self n.
	sortlist initarr: self!

littlest: x littlest _ x!

n ^QSortelements!

run
	self initarr.
	sortlist sort.
	((sortlist at: 1) ~= littlest or: [(sortlist at: self n ) ~= biggest]) ifTrue: [self error: 'error']
	"Quicksort2Benchmark new run"! !

"*****************************************************************************"

AbstractBenchmark subclass: #NestedLoopBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!NestedLoopBenchmark methodsFor: 'misc'!

nestedLoopTest
	| sum |
	sum _ 0.
	1 to: 100 do: [ :i | 1 to: 100 do: [ :j | sum _ sum + 1 ] ].
	^ sum!

run
	"NestedLoopBenchmark new run"
	100 timesRepeat: [ self nestedLoopTest ]! !

"*****************************************************************************"

AbstractBenchmark subclass: #TowersBenchmark
	instanceVariableNames: 'stack cellspace freelist movesdone '
	classVariableNames: 'Maxcells Stackrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TowersBenchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^190].
	mach = 'sun3_160' ifTrue: [^1130].
 mach = 'macIIcx'  ifTrue: [^1134]!

fixedBase
	^2.39! !

!TowersBenchmark methodsFor: 'run'!

run
	cellspace _ Array new: Maxcells.
	1 to: cellspace size do: [:i | cellspace at: i put: TowersBenchmarkElement new].
	stack _ Array new: Stackrange. 
	1 to: Maxcells do: [:i | (cellspace at: i)
			next: i - 1].
	freelist _ Maxcells.
	self init: 1 to: 14.
	self makenull: 2.
	self makenull: 3.
	movesdone _ 0.
	self
		towerI: 1
		j: 2
		k: 14.
	movesdone = 16383 ifFalse: [self error: ' Error in towers.']! !

!TowersBenchmark methodsFor: 'private'!

error: msg
	super error: 'Error in towers: ', msg!

getelement
	| temp |
	freelist > 0
		ifTrue: 
			[temp _ freelist.
			freelist _ (cellspace at: freelist) next]
		ifFalse: [self error: 'out of space'].
	^temp!

init: s to: n 
	self makenull: s.
	n
		to: 1
		by: -1
		do: [:discctr | self push: discctr onto: s]!

makenull: s
	stack at: s put: 0!

moveFrom: s1 to: s2 
	self push: (self pop: s1)
		onto: s2.
	movesdone _ movesdone + 1!

pop: s 
	| temp1 temp |
	(stack at: s)
		> 0 ifFalse: [^self error: 'nothing to pop'].
	temp1 _ (cellspace at: (stack at: s)) discsize.
	temp _ (cellspace at: (stack at: s)) next.
	(cellspace at: (stack at: s))
		next: freelist.
	freelist _ stack at: s.
	stack at: s put: temp.
	^temp1!

push: i onto: s 
	| localel |
	(stack at: s)
		> 0 ifTrue: [(cellspace at: (stack at: s)) discsize <= i ifTrue: [^self error: 'disc size error']].
	localel _ self getelement.
	(cellspace at: localel)
		next: (stack at: s).
	stack at: s put: localel.
	(cellspace at: localel)
		discsize: i!

towerI: i j: j k: k 
	| other |
	k = 1
		ifTrue: [self moveFrom: i to: j]
		ifFalse: 
			[other _ 6 - i - j.
			self
				towerI: i
				j: other
				k: k - 1.
			self moveFrom: i to: j.
			self
				towerI: other
				j: j
				k: k - 1]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- Commented out by GVG
TowersBenchmark class
	instanceVariableNames: ''
 -----"!

!TowersBenchmark class methodsFor: 'constants'!

initialize
    Maxcells _ 18. Stackrange _ 3 "TowersBenchmark initialize"! !

TowersBenchmark initialize!

"*****************************************************************************"

QuicksortBenchmark subclass: #TreeSortBenchmark
	instanceVariableNames: 'tree '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSortBenchmark methodsFor: 'run'!

run
	self initarr.
	tree _ TreeSortNodeBenchmark new val: (sortlist at: 1).
	2 to: QSortelements do: [:i | self insert: (sortlist at: i)
			Into: tree].
	(self checkTree: tree)
		ifFalse: [self error: ' Error in Tree.']! !

!TreeSortBenchmark methodsFor: 'private'!

checkTree: p 
	| result |
	result _ true.
	p left notNil ifTrue: [p left val <= p val
			ifTrue: [result _ false]
			ifFalse: [result _ (self checkTree: p left)
							and: [result]]].
	p right notNil ifTrue: [p right val >= p val
			ifTrue: [result _ false]
			ifFalse: [result _ (self checkTree: p right)
							and: [result]]].
	^result!

createNode: n
	^TreeSortNodeBenchmark new val: n!

insert: n Into: t 
	n > t val
		ifTrue: [t left isNil
				ifTrue: [t left: (self createNode: n)]
				ifFalse: [self insert: n Into: t left]]
		ifFalse: [n < t val ifTrue: [t right isNil
					ifTrue: [t right: (self createNode: n)]
					ifFalse: [self insert: n Into: t right]]]! !

!TreeSortBenchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^780].
	mach = 'sun3_160' ifTrue: [^1160].
 mach = 'macIIcx'  ifTrue: [^1183]!

fixedBase ^2.5! !

"*****************************************************************************"

AbstractBenchmark subclass: #PermBenchmark
	instanceVariableNames: 'permarray pctr '
	classVariableNames: 'Permcount Permrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!PermBenchmark methodsFor: 'running'!

run
	
	pctr _ 0.
	Permcount
		timesRepeat: 
			[self initialize.
			self permute: 7].
	pctr ~= (8660 *  Permcount) ifTrue: [self error: 'Error in perm']! !

!PermBenchmark methodsFor: 'constants'!

cTimes: mach
 mach = 'sun4_260' ifTrue: [^110].
 mach = 'sun3_160' ifTrue: [^748].
 mach = 'macIIcx'  ifTrue: [^1017]!

fixedBase ^ 1.75!

shortPrintString ^'perm'! !

!PermBenchmark methodsFor: 'initialization'!

initialize
	permarray _ Array new: Permrange.
	1 to: permarray size do: [:i | permarray at: i put: i - 1]! !

!PermBenchmark methodsFor: 'private'!

permute: n 
	pctr _ pctr + 1.
	n ~= 1
		ifTrue: 
			[self permute: n - 1.
			n - 1
				to: 1
				by: -1
				do: 
					[:k | 
					self swap: n with: k.
					self permute: n - 1.
					self swap: n with: k]]!

swap: a with: b 
	| t |
	t _ permarray at: a.
	permarray at: a put: (permarray at: b).
	permarray at: b put: t! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
PermBenchmark class
	instanceVariableNames: ''
 -----"!

!PermBenchmark class methodsFor: 'constants'!

initialize 
    Permcount _ 5. Permrange _ 10 "PermBenchmark initialize"! !

PermBenchmark initialize!


"*****************************************************************************"

PermBenchmark subclass: #Perm2Benchmark		"GVG Was AbstractBenchmark..."
	instanceVariableNames: ''
	classVariableNames: '' "<---Permcount Permrange deleted by GVG"
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Perm2Benchmark methodsFor: 'misc'!

run | permarray |
	permarray _  PermArray new: Permrange.
	permarray pctr: 0.
	Permcount timesRepeat: [permarray permInitialize. permarray permute: 7].
	permarray pctr ~= (8660 * Permcount) ifTrue: [ self error: 'error'].
	"Perm2Benchmark new run"! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
Perm2Benchmark class
	instanceVariableNames: ''
 -----"!

!Perm2Benchmark class methodsFor: 'constants'!

initialize Permcount _ 5. Permrange _ 10 "Perm2Benchmark initialize"! !

Perm2Benchmark initialize!

"*****************************************************************************"

AbstractBenchmark subclass: #SumFromToBenchmark
	instanceVariableNames: 'x '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!

!SumFromToBenchmark methodsFor: 'misc'!

run
	x _0.
	100 timesRepeat: [self sumFrom: 1 to: 10000]	
	"SumFromToBenchmark new run"!

sumFrom: start to: end
	| sum |
	sum _ 0.
	start to: end do: [:i | sum _ sum + i].
	^sum! !

"*****************************************************************************"

AbstractBenchmark subclass: #IntMMBenchmark
	instanceVariableNames: 'ma mb mr '
	classVariableNames: 'IRowsize '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMMBenchmark methodsFor: 'run'!

run
	self initialize.
	mr _ self mmMatrix.
	ma _ self initmatrix.
	mb _ self initmatrix.
	self initrand.
	1 to: IRowsize do: [:i | 1 to: IRowsize do: [:j | (mr at: i)
				at: j put: (self
					innerproductOf: ma
					and: mb
					row: i
					column: j)]]! !

!IntMMBenchmark methodsFor: 'private'!

initmatrix 
	| m temp |
	m _ self mmMatrix.
	1 to: IRowsize do: [:i | 1 to: IRowsize do: 
			[:j | 
			temp _ self newValue.
			(m at: i)
				at: j put: temp - (temp // 120 * 120) - 60]].
	^m!

innerproductOf: a and: b row: row column: column 
	| result |
	result _ 0.
	1 to: IRowsize do: [:i | result _ ((a at: row)
						at: i) * ((b at: i) at: column) + result].
	^result!

makeMatrix: n by: m 
	| r |
	r _ Array new: n.
	1 to: r size do: [:i | r at: i put: (Array new: m)].
	^r!

makePuzzleMatrix: n by: m 
	| r |
	r _ PuzzleArray new: n.
	0 to: r size - 1 do: [:i | r at: i put: (PuzzleArray new: m)].
	^r!

mmMatrix
	^self makeMatrix: IRowsize + 1 by: IRowsize + 1!

newValue
	^self rand! !

!IntMMBenchmark methodsFor: 'constants'!

cTimes: mach
 mach = 'sun4_260' ifTrue: [^270].
 mach = 'sun3_160' ifTrue: [^928].
 mach = 'macIIcx'  ifTrue: [^400]!

fixedBase ^1.46! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
IntMMBenchmark class
	instanceVariableNames: ''
 -----"!

!IntMMBenchmark class methodsFor: 'constants'!

initialize 
    IRowsize _ 40 "IntMMBenchmark initialize"! !

IntMMBenchmark initialize!


"*****************************************************************************"

Quicksort2Benchmark subclass: #TreeSort2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TreeSort2Benchmark methodsFor: 'misc'!

run
	| tree |
	self initarr.
	tree _ TreeSort2Node new contents: (sortlist at: 1).
	2 to: self n do: [:i | tree insert: (sortlist at: i)].
	tree checkTree or: [self error: 'error'].
	"TreeSort2Benchmark new run"! !

!TreeSort2Benchmark methodsFor: 'constants'!

cTimes: mach
	mach = 'sun4_260' ifTrue: [^780].
	mach = 'sun3_160' ifTrue: [^1160].
 mach = 'macIIcx'  ifTrue: [^1183]!

fixedBase ^2.5! !

"*****************************************************************************"

AbstractBenchmark subclass: #QueensBenchmark
	instanceVariableNames: ''
	classVariableNames: 'Queencount '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!QueensBenchmark methodsFor: 'run'!

run
	Queencount timesRepeat: [self doQueens]! !

!QueensBenchmark methodsFor: 'private'!

doQueens
	| i a b c x |
	a _ Array new: 9.
	b _ Array new: 17.
	c _ Array new: 15.
	x _ Array new: 9.
	i _ -7.
	[i <= 16]
		whileTrue: 
			[(i >= 1 and: [i <= 8])
				ifTrue: [a at: i put: true].
			i >= 2 ifTrue: [b at: i put: true].
			i <= 7 ifTrue: [c at: i + 8 put: true].
			i _ i + 1].
	(self
		tryI: 1
		A: b
		B: a
		C: c
		X: x)
		ifFalse: [self error: 'Error in queens']!

tryI: i A: a B: b C: c X: x 
	| j q |
	j _ 0.
	q _ false.
	[q not and: [j ~= 8]]
		whileTrue: 
			[j _ j + 1.
			q _ false.
			((b at: j)
				and: [(a at: i + j)
						and: [c at: i - j + 8]])
				ifTrue: 
					[x at: i put: j.
					b at: j put: false.
					a at: i + j put: false.
					c at: i - j + 8 put: false.
					i < 8
						ifTrue: 
							[q _ self
										tryI: i + 1
										A: a
										B: b
										C: c
										X: x.
							q
								ifFalse: 
									[b at: j put: true.
									a at: i + j put: true.
									c at: i - j + 8 put: true]]
						ifFalse: [q _ true]]].
	^q! !

!QueensBenchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^90].
	mach = 'sun3_160' ifTrue: [^532].
 mach = 'macIIcx'  ifTrue: [^300]!

fixedBase ^1.83! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
QueensBenchmark class
	instanceVariableNames: ''
 -----"!

!QueensBenchmark class methodsFor: 'constants'!

initialize Queencount _ 50 "QueensBenchmark initialize"! !

QueensBenchmark initialize!


"*****************************************************************************"

IntMMBenchmark subclass: #PuzzleBenchmark
	instanceVariableNames: 'piececount class piecemax puzzl p n kount '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!PuzzleBenchmark methodsFor: 'private'!

fitI: i J: j 
	0 to: (piecemax at: i)
		do: [:k | (  ((p at: i) at: k) 
				and: [(puzzl at: j + k) ])
				ifTrue: [^false]].
	^true!

placeI: i J: j 
	0 to: (piecemax at: i)
		do: [:k | ((p at: i) at: k)
				ifTrue: [puzzl at: j + k put: true]].
	piececount at: (class at: i)
		put: (piececount at: (class at: i)) - 1.
	j to: self puzzleSize do: [:k | (puzzl at: k) ifFalse: [^k]].
	^0!

removeI: i J: j 
	0 to: (piecemax at: i)
		do: [:k | ((p at: i) at: k)
				ifTrue: [puzzl at: j + k put: false]].
	piececount at: (class at: i)
		put: (piececount at: (class at: i))
				+ 1!

run1
	piececount _ PuzzleArray new: self classmax + 1.
	class _ PuzzleArray new: self typemax + 1.
	piecemax _ PuzzleArray new: self typemax + 1.
	puzzl _ PuzzleArray new: self puzzleSize + 1.
	p _ self makePuzzleMatrix: self typemax + 1 by: self puzzleSize + 1.
	0 to: self puzzleSize do: [:m | puzzl at: m put: true]!

run2  
	1 to: 5 do: [ :i |
	    1 to: 5 do: [ :j |
	        1 to: 5 do: [ :k |
		    puzzl at: i + (self d * (j + (self d * k))) put: false ]]].

	0 to: self typemax   do: [ :i |
	    0 to: self puzzleSize   do: [ :m |
		(p at: i) at: m put: false]].

	0 to: 3 do: [ :i | 0 to: 1 do: [ :j | 0 to: 0 do: [ :k | (p at: 0) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 0 put: 0.
	piecemax at: 0 put: 3 + (self d * 1) + (self d * self d * 0).

	0 to: 1 do: [ :i | 0 to: 0 do: [ :j | 0 to: 3 do: [ :k | (p at: 1) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 1 put: 0.
	piecemax at: 1 put: 1 + (self d * 0) + (self d * self d * 3).

	0 to: 0 do: [ :i | 0 to: 3 do: [ :j | 0 to: 1 do: [ :k | (p at: 2) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 2 put: 0.
	piecemax at: 2 put: 0 + (self d * 3) + (self d * self d * 1).

	0 to: 1 do: [ :i | 0 to: 3 do: [ :j | 0 to: 0 do: [ :k | (p at: 3) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 3 put: 0.
	piecemax at: 3 put: 1 + (self d * 3) + (self d * self d * 0).

	0 to: 3 do: [ :i | 0 to: 0 do: [ :j | 0 to: 1 do: [ :k | (p at: 4) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 4 put: 0.
	piecemax at: 4 put: 3 + (self d * 0) + (self d * self d * 1).

	0 to: 0 do: [ :i | 0 to: 1 do: [ :j | 0 to: 3 do: [ :k | (p at: 5) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 5 put: 0.
	piecemax at: 5 put: 0 + (self d * 1) + (self d * self d * 3).

	0 to: 2 do: [ :i | 0 to: 0 do: [ :j | 0 to: 0 do: [ :k | (p at: 6) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 6 put: 1.
	piecemax at: 6 put: 2 + (self d * 0) + (self d * self d * 0).

	0 to: 0 do: [ :i | 0 to: 2 do: [ :j | 0 to: 0 do: [ :k | (p at: 7) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 7 put: 1.
	piecemax at: 7 put: 0 + (self d * 2) + (self d * self d * 0).

	0 to: 0 do: [ :i | 0 to: 0 do: [ :j | 0 to: 2 do: [ :k | (p at: 8) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 8 put: 1.
	piecemax at: 8 put: 0 + (self d * 0) + (self d * self d * 2).

	0 to: 1 do: [ :i | 0 to: 1 do: [ :j | 0 to: 0 do: [ :k | (p at: 9) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 9 put: 2.
	piecemax at: 9 put: 1 + (self d * 1) + (self d * self d * 0).

	0 to: 1 do: [ :i | 0 to: 0 do: [ :j | 0 to: 1 do: [ :k | (p at: 10) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 10 put: 2.
	piecemax at: 10 put: 1 + (self d * 0) + (self d * self d * 1).

	0 to: 0 do: [ :i | 0 to: 1 do: [ :j | 0 to: 1 do: [ :k | (p at: 11) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 11 put: 2.
	piecemax at: 11 put: 0 + (self d * 1) + (self d * self d * 1).

	0 to: 1 do: [ :i | 0 to: 1 do: [ :j | 0 to: 1 do: [ :k | (p at: 12) at: i + (self d * (j + (self d * k))) put: true ]]].
	class at: 12 put: 3.
	piecemax at: 12 put: 1 + (self d * 1) + (self d * self d * 1).!

trial: j 
	| k |
	kount _ kount + 1.
	0 to: self typemax do: [:i | (piececount at: (class at: i))
			~= 0 ifTrue: [(self fitI: i J: j)
				ifTrue: 
					[k _ self placeI: i J: j.
					((self trial: k)
						or: [k = 0])
						ifTrue: [^true]
						ifFalse: [self removeI: i J: j]]]].
	^false! !

!PuzzleBenchmark methodsFor: 'constants'!

classmax ^3!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^760].
	mach = 'sun3_160' ifTrue: [^5726].
 mach = 'macIIcx'  ifTrue: [^1900]!

d ^8!

fixedBase ^0.5!

puzzleSize ^511!

typemax ^12! !

!PuzzleBenchmark methodsFor: 'run'!

run
	| m |
	n _ 0.
	kount _ 0.
	self run1; run2.
	piececount at: 0 put: 13.
	piececount at: 1 put: 3.
	piececount at: 2 put: 1.
	piececount at: 3 put: 1.
	m _ 1 + (self d * (1 + (self d * 1))).
	kount _ 0.
	(self fitI: 0 J: m) ifTrue: [n _ self placeI: 0 J: m] ifFalse: [self error: 'error1 in puzzle'].
	(self trial: n ) ifFalse: [self error: 'error2 in puzzle'].
	kount = 2005 ifFalse: [self error: 'error3 in puzzle']! !

"*****************************************************************************"

AbstractBenchmark subclass: #SieveBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!SieveBenchmark methodsFor: 'misc'!

run
	| size iter flags prime k |
	size _ 8190.
	iter _ 10.
	flags _ (Array new: size + 1)
				atAllPut: true.
	iter timesRepeat: [
		0 to: size do: [:i | (flags at: i + 1) ifTrue: [
			prime _ i + i + 3.
			k _ i + prime.
			[k <= size] whileTrue: [
				flags at: k put: false.
				k _ k + prime]]]]
	"SieveBenchmark new run"! !

IntMMBenchmark subclass: #IntMM2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IntMM2Benchmark methodsFor: 'run'!

run
	| m |
	self initrand.
	m _ ((IntMM2Matrix new: IRowsize by: IRowsize) init: self) * ((IntMM2Matrix new: IRowsize by: IRowsize) init: self).
	^m
	"IntMM2Benchmark new run"! !

"*****************************************************************************"

AbstractBenchmark subclass: #AtAllPutBenchmark
	instanceVariableNames: ''
	classVariableNames: 'Vect '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!AtAllPutBenchmark methodsFor: 'misc'!

run
	"AtAllPutBenchmark new run"
	Vect atAllPut: 7! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
AtAllPutBenchmark class
	instanceVariableNames: ''
 -----"!

!AtAllPutBenchmark class methodsFor: 'initialize'!

initialize
	"AtAllPutBenchmark initialize"
	Vect _ Array new: 100000! !

AtAllPutBenchmark initialize!


"*****************************************************************************"

AbstractBenchmark subclass: #Queens2Benchmark
	instanceVariableNames: 'freeMajs freeRows freeMins qrows '
	classVariableNames: 'Queencount '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Queens2Benchmark methodsFor: 'run'!

run
	Queencount timesRepeat: [self initialize doQueens]
	"Queens2Benchmark new run"! !

!Queens2Benchmark methodsFor: 'private'!

doQueens
	(self try: 1) 
		ifFalse: [self error: 'Error in queens']!

initialize
	freeMajs _ (Array new: 15) atAllPut: true.
	freeRows _ (Array new: 8) atAllPut: true.
	freeMins _ (Array new: 15) atAllPut: true.
	qrows _ Array new: 8.!

try: col
	1 to: 8 do: [ :row |
		((freeRows at: row) and: [(freeMajs at: col + row - 1) and: [freeMins at: col - row + 7]]) ifTrue: [
			qrows at: col put: row.
			freeRows at: row put: false.
			freeMajs at: col + row - 1 put: false.
			freeMins at: col - row + 7 put: false.
			8 = col ifTrue: [ ^true ].
			(self try: col + 1 ) ifTrue: [^true].
			freeRows at: row put: true.
			freeMajs at: col + row - 1 put: true.
			freeMins at: col - row + 7 put: true] ].
	^false! !

!Queens2Benchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^90].
	mach = 'sun3_160' ifTrue: [^532].
 mach = 'macIIcx'  ifTrue: [^300]!

fixedBase ^1.83! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
Queens2Benchmark class
	instanceVariableNames: ''
 -----"!

!Queens2Benchmark class methodsFor: 'constants'!

initialize Queencount _ 50 "Queens2Benchmark initialize"! !

Queens2Benchmark initialize!


"*****************************************************************************"

IntMMBenchmark subclass: #MMBenchmark
	instanceVariableNames: ''
	classVariableNames: 'Rowsize '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!MMBenchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^320].
	mach = 'sun3_160' ifTrue: [^6718].
 mach = 'macIIcx'  ifTrue: [^817] "18983 if not optimized for 68881"!

fixedBase ^0.73!

floatBase ^2.92! !

!MMBenchmark methodsFor: 'private'!

initmatrix 
	| m temp |
	m _ self mmMatrix.
	1 to: Rowsize do: [:i | 1 to: Rowsize do: 
			[:j | 
			temp _ self newValue.
			(m at: i)
				at: j put: temp - (temp / 120 * 120) - 60]].
	^m!

newValue
	^super newValue asFloat! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
MMBenchmark class
	instanceVariableNames: ''
 -----"!

!MMBenchmark class methodsFor: 'constants'!

initialize Rowsize _ 40 "MMBenchmark initialize"! !

MMBenchmark initialize!

"*****************************************************************************"

AtAllPutBenchmark subclass: #SumAllBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!SumAllBenchmark methodsFor: 'misc'!

run
	| sum |
	sum _ 0.
	Vect do: [ :i | sum _ sum + 1 ].
	^ sum! !

"*****************************************************************************"

Quicksort2Benchmark subclass: #BubbleSort2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BubbleSort2Benchmark methodsFor: 'misc'!

arrayClass ^BubbleSort2Array!

n ^500! !

!BubbleSort2Benchmark methodsFor: 'constants'!

cTimes: mach
	mach = 'sun4_260' ifTrue: [^180].
	mach = 'sun3_160' ifTrue: [^942].
 mach = 'macIIcx'  ifTrue: [^633]!

fixedBase ^ 1.61! !

"*****************************************************************************"

AbstractBenchmark subclass: #TaklBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!TaklBenchmark methodsFor: 'misc'!

listn: n
	0 = n ifTrue: [ ^ nil ] ifFalse: [ ^ (Cons new car: n) cdr: (self listn: n - 1) ]!

run
	"TaklBenchmark new run"
	| r |
	r _ self taklX: (self listn: 18) Y: (self listn: 12) Z: (self listn: 6).
	(r = (self listn: 7)) ifFalse: [ self error: 'takl results are wrong' ]!

shorterpX: x Y: y
	^ (nil ~= y) and: [ (nil = x) or: [ self shorterpX: x cdr Y: y cdr ] ]!

taklX: x Y: y Z: z
	(self shorterpX: y Y: x) ifFalse: [^z]
		ifTrue: [^self
				taklX: (self
						taklX: x cdr
						Y: y
						Z: z)
				Y: (self
						taklX: y cdr
						Y: z
						Z: x)
				Z: (self
						taklX: z cdr
						Y: x
						Z: y)]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
TaklBenchmark class
	instanceVariableNames: ''
 -----"!

"*****************************************************************************"

AtAllPutBenchmark subclass: #IncrementAllBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!IncrementAllBenchmark methodsFor: 'misc'!

run
	"IncrementAllBenchmark new run"
	1 to: Vect size do: [ :i | Vect at: i put: (Vect at: i) + 1 ]! !

"*****************************************************************************"

QuicksortBenchmark subclass: #BubbleSortBenchmark
	instanceVariableNames: 'top '
	classVariableNames: 'Sortelements SortElements '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!BubbleSortBenchmark methodsFor: 'run'!

initarr
	| temp |
	self initrand.
	biggest _ -1000000.
	littlest _ 1000000.
	sortlist _ Array new: Sortelements.
	1 to: Sortelements do: 
		[:i | 
		temp _ self rand.
		sortlist at: i put: temp - (temp // 100000 * 100000) - 50000.
		(sortlist at: i)
			> biggest ifTrue: [biggest _ sortlist at: i].
		(sortlist at: i)
			< littlest ifTrue: [littlest _ sortlist at: i]]!

run   | i  j |
	self initarr.
	top _    Sortelements.
	[top > 1] whileTrue: [
	    i_ 1.
	    [i < top] whileTrue: [
		(sortlist at: i) > (sortlist at: i + 1) ifTrue: [
		    j_ sortlist at: i.
		    sortlist at: i put: (sortlist at: i + 1).
		    sortlist at: i + 1 put: j.
		].
		i_ i + 1.
	    ].
	    top_ top - 1.
	].
	((sortlist at: 1) ~= littlest or: 
	[(sortlist at: Sortelements) ~= biggest]) ifTrue:
	    [self error: 'Error3 in Bubble'  ]! !

!BubbleSortBenchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^180].
	mach = 'sun3_160' ifTrue: [^942].
 mach = 'macIIcx'  ifTrue: [^633]!

fixedBase ^ 1.61! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
BubbleSortBenchmark class
	instanceVariableNames: ''
 -----"!

!BubbleSortBenchmark class methodsFor: 'constants'!

initialize Sortelements _ 500 "BubbleSortBenchmark initialize"! !

BubbleSortBenchmark initialize!

"*****************************************************************************"

IntMM2Benchmark subclass: #MM2Benchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!MM2Benchmark methodsFor: 'misc'!

newValue ^super newValue asFloat! !

!MM2Benchmark methodsFor: 'constants'!

cTimes: mach
	mach = 'sun4_260' ifTrue: [^320].
	mach = 'sun3_160' ifTrue: [^6718].
 mach = 'macIIcx'  ifTrue: [^817] "18983 if not optimized for 68881"!

fixedBase ^0.73!

floatBase ^2.92! !



"*****************************************************************************"

AbstractBenchmark subclass: #Towers2Benchmark
	instanceVariableNames: 'stack movesdone '
	classVariableNames: 'Maxcells Sentinel Stackrange '
	poolDictionaries: ''
	category: 'Self-Benchmarks'!


!Towers2Benchmark methodsFor: 'constants'!

cTimes: mach 
	mach = 'sun4_260' ifTrue: [^190].
	mach = 'sun3_160' ifTrue: [^1130].
 mach = 'macIIcx'  ifTrue: [^1134]!

fixedBase
	^2.39! !

!Towers2Benchmark methodsFor: 'run'!

makenull: s
	stack at: s put: Sentinel!

run
	stack _ Array new: Stackrange.
	self makenull: 1.
	self makenull: 2.
	self makenull: 3.
	self init: 1 to: 14.
	movesdone _ 0. 
	self
		towerI: 1
		j: 2
		k: 14.
	movesdone = 16383 ifFalse: [self error: ' Error in towers.'].
	"Towers2Benchmark new run"! !

!Towers2Benchmark methodsFor: 'private'!

error: msg
	super error: 'Error in towers: ', msg! !

!Towers2Benchmark methodsFor: 'misc'!

init: s to: n 

	n
		to: 1
		by: -1
		do: [:discctr | self push: (Towers2Disk new discSize: discctr) onto: s]!

moveFrom: s1 to: s2 
	self push: (self pop: s1)
		onto: s2.
	movesdone _ movesdone + 1!

pop: s 
	| result |
	Sentinel == (stack at: s) ifTrue: [self error: 'empty'].
	result _ stack at: s.
	stack at: s put: result next.
	^result!

push: d onto: s 
	| localel |
	localel _ stack at: s.
	d >= localel ifTrue: [^self error: 'disc size error'].
	stack at: s put: (d next: localel)!

towerI: i j: j k: k 
	| other |
	k = 1
		ifTrue: [self moveFrom: i to: j]
		ifFalse: 
			[other _ 6 - i - j.
			self
				towerI: i
				j: other
				k: k - 1.
			self moveFrom: i to: j.
			self
				towerI: other
				j: j
				k: k - 1]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "

"----- GVG
Towers2Benchmark class
	instanceVariableNames: ''
 -----"!

!Towers2Benchmark class methodsFor: 'constants'!

initialize Maxcells _ 18. Stackrange _ 3. Sentinel _ Towers2Disk new discSize: 15. "Towers2Benchmark initialize"! !

Towers2Benchmark initialize!
