# ----------------------------- bench.py ----------------------------------

# Python version of Stanford Integer Benchmarks

import time
import string

# -------------------------- Module Variables ------------------------------

#Constants
true = 1
false = 0

#Variables
seed = 74755

# ------------------------- Utility Functions ------------------------------

def Initrand():
	#Initialize seed for random number generator
	global seed
	seed = 74755

def Rand():
	#A very poor random number generator
	global seed
	seed = (seed * 1309 + 13849) & 65535
	return seed

def MakeMatrix(rows, cols):
	#Make a two-dimensional matrix
	#Note: "row" and "col" designation for 1st/2nd parameter is arbitrary
	mat = []
	for i in range (rows):
		mat.append([None]*cols)
	return mat

# ---------------------------- Bubble Sort ---------------------------------

bSortElements = 500 

def Bubble():
    # "Bubble sort benchmark - creates and sorts an array of size bSortElements."
	size = bSortElements
	sortlist = InitSortlist(size)
	top = size - 1
	while top > 1:
		i = 0
		while i < top:
			if sortlist[i] > sortlist[i+1]:
				j = sortlist[i]
				sortlist[i] = sortlist[i+1]
				sortlist[i+1] = j
			i = i + 1
		top = top - 1
	if sortlist[0] != min(sortlist) or sortlist[size-1] != max(sortlist):
		print "Error in Quicksort"

def InitSortlist(size):
	sortlist = [None]*size;
	for i in range(size):
		sortlist[i] = Rand()
	return sortlist


# ---------------------------- Quicksort ---------------------------------

qSortElements = 5000

def Quick():
	size = qSortElements
	sortlist = InitSortlist(size)
	Quicksort(sortlist, 0, size-1)
	if sortlist[0] != min(sortlist) or sortlist[size-1] != max(sortlist):
		print "Error in Quicksort"

def Quicksort(a, l, r):
	i = l
	j = r
	x = a[(l+r)/2]
	if i > j:
		raise "I didn't expect this"
	while i <= j:
		while a[i] < x: i=i+1
		while x < a[j]: j=j-1
		if i <= j:
			w = a[i]
			a[i] = a[j]
			a[j] = w
			i = i+1
			j = j-1
	if l < j: Quicksort(a, l, j)
	if i < r: Quicksort(a, i, r)

# InitSortlist is in Bubble Sort


# ---------------------------- Treesort ----------------------------------

treeSortElements = 5000

class TreeSortNode:
	# This represents a structure will 3 variables
	#   left - reference to previous TreeSortNode
	#   right - reference to next TreeSortNode
	#   value - actual value contained in the node
	# All variables are publically accessable 
	def __init__(self, val):
		self.left = None
		self.right = None
		self.value = val

def Tree():
	sortlist = InitSortlist(treeSortElements)
	tree = TreeSortNode(sortlist[0])
	for i in range(1, treeSortElements):
		InsertTree(sortlist[i], tree)
	if not CheckTree(tree):
		print "Error in TreeSort"

def InsertTree(n, tree):
	# ERROR: This will fail to insert a node if n matches
	#        an existing value.  This is the same in all versions.
	# Note: left gets the greater value (strange)
	if n > tree.value:
		if tree.left:
			InsertTree(n, tree.left)
		else:
			tree.left = TreeSortNode(n)
	elif n < tree.value:
		if tree.right:
			InsertTree(n, tree.right)
		else:
			tree.right = TreeSortNode(n)

def CheckTree(node):
	result = 1
	if node.left:
		if node.left.value < node.value:
			result = 0
		else:
			result = CheckTree(node.left) and result
	if node.right:
		if node.right.value > node.value:
			result = 0
		else:
			result = CheckTree(node.right) and result
	return result

# InitSortlist defined in BubbleSort

# ---------------------------- Fast Sort ---------------------------------

# Fast sort uses languages built-in sorting function

def Fast():
	size = qSortElements
	sortlist = InitSortlist(size)
	FastSort(sortlist)
	if sortlist[0] != min(sortlist) or sortlist[size-1] != max(sortlist):
		print "Error in Fastsort"

def FastSort(sortlist):
	sortlist.sort()
	
def FastInit():
	"Initialize only to permit FastSort to be run independently"
	return InitSortlist(qSortElements)

# InitSortlist is in Bubble Sort

# ----------------- Integer Matrix Multiply (List) ----------------------

# Integer Matrix Multiply (IntMM) using a list of lists

# Uses 0-based indexing instead of the 1 based indexing
# used in the C and Smalltalk versions.

iRowSize = 40

def IntMM():
	imr = mmMatrix(iRowSize)
	ima = InitMatrix(iRowSize)
	imb = InitMatrix(iRowSize)
	Initrand()
	for i in range(iRowSize):
		for j in range(iRowSize):
			imr[i][j] = Innerproduct(ima, imb, i, j)

def Innerproduct(a, b, row, column):
	result = 0
	for i in range(iRowSize):
		result = result + a[row][i] + b[i][column]
	return result

def mmMatrix(size):
	return MakeMatrix(size, size)

def InitMatrix(size):
	m = mmMatrix(size)
	for i in range(size):
		for j in range(size):
			temp = Rand()
			m[i][j] = temp - (temp/120)*120 - 60
	return m

# --------------------- Matrix Multiply (List) --------------------------

# Floating pint Matrix Multiply (IntMM) using a list of lists

# Uses 0-based indexing instead of the 1 based indexing
# used in the C and Smalltalk versions.

rowSize = 40

def MM():
	mr = mmMatrix(rowSize)
	ma = rInitMatrix(rowSize)
	mb = rInitMatrix(rowSize)
	Initrand()
	for i in range(rowSize):
		for j in range(rowSize):
			mr[i][j] = rInnerproduct(ma, mb, i, j)

def rInnerproduct(a, b, row, column):
	result = 0.0
	for i in range(rowSize):
		result = result + a[row][i] + b[i][column]
	return result

# Use mmMatrix from IntMM

def rInitMatrix(size):
	m = mmMatrix(size)
	for i in range(size):
		for j in range(size):
			temp = Rand()
			m[i][j] = temp - (temp/120)*120 - 60
	return m

# -------------------- Permutation (using globals) -------------------------

# Permutation program, heavily recursive, written by Denny Brown.

pctr = 0
permarray = [0]

def PermG():
	global pctr
	pctr = 0
	for i in range(5):
		InitPermG()
		PermuteG(7)
	if pctr != 43300:
		print "Error in Perm."

def PermuteG(n):
	global pctr, permarray
	pctr = pctr + 1
	if n != 1:
		PermuteG(n-1)
		k = n - 1
		while k >= 1:
			SwapG(permarray, n, k)
			PermuteG(n - 1)
			SwapG(permarray, n, k)
			k = k - 1

def SwapG(list, index1, index2):
	t = list[index1]
	list[index1] = list[index2]
	list[index2] = t

def InitPermG():
	global permarray
	permarray = [0]            #C code starts at index 1
	for i in range(7):
		permarray.append(i)
	

# --------------------- Permutation (using locals) -------------------------

# Permutation program, heavily recursive, written by Denny Brown.

pctr = 0

def PermL():
	global pctr
	pctr = 0
	for i in range(5):
		permarray = InitPermL()
		PermuteL(permarray, 7)
	if pctr != 43300:
		print "Error in Perm."

def PermuteL(permarray, n):
	global pctr
	pctr = pctr + 1
	if n != 1:
		PermuteL(permarray, n-1)
		k = n - 1
		while k >= 1:
			Swap(permarray, n, k)
			PermuteL(permarray, n - 1)
			Swap(permarray, n, k)
			k = k - 1

def Swap(list, index1, index2):
	t = list[index1]
	list[index1] = list[index2]
	list[index2] = t

def InitPermL():
	permarray = [0]            #C code starts at index 1
	for i in range(7):
		permarray.append(i)
	return permarray


# ----------------------------- 8 Queens -----------------------------------

def Queens():
	for i in range(50): 
		DoQueens()

def DoQueens():
	a = [false] * 9
	b = [false] * 17
	c = [false] * 15
	x = [false] * 9
	for i in range(-7, 17):
		if i >= 1 and i <= 8:
			a[i] = true
		if i >= 2:
			b[i] = true
		if i <= 7:
			c[i+7] = true
	q = Try(1, b, a, c, x)
	if not q:
		print "Error in Queens."

def Try(i, a, b, c, x):
	"Try a position"
	j = 0
	q = false
	while (not q) and j != 8:
		j = j + 1
		q = false
		if b[j] and a[i+j] and c[i-j+7]:
			x[i] = j
			b[j] = false
			a[i+j] = false
			c[i-j+7] = false
			if i < 8:
				q = Try(i+1, a, b, c, x)
				if not q:
					b[j] = true
					a[i+j] = true
					c[i-j+7] = true
			else:
				q = true
	return q


# ------------------------- Towers of Hanoi --------------------------------

#Constants
maxcells = 18
stackrange = 3

#Global variables
stack = []
freelist = 0
movesdone = 0

class Element:
	next = 0
	discsize = 0

cellspace = [None]    #Will contain elements

#Functions

def InitCellspace():
	global cellspace, maxcells
	cellspace = [None]
	for i in range(maxcells):
		cellspace.append(Element())

def InitStack():
	for i in range(stackrange + 1):
		stack.append(0)

def Error(s):
	print "Towers error: ", s
	raise s

def Makenull(s):
	global stack
	stack[s] = 0

def Getelement():
	global freelist, cellspace
	temp = freelist
	if freelist > 0:
		freelist = cellspace[freelist].next
	else:
		Error("out of space   ")
	return temp

def Push(i, s):
	global cellspace, stack
	errorfound = false
	if stack[s] > 0:
		if cellspace[stack[s]].discsize <= i:
			errorfound = true
			Error("disc size error")
	if not errorfound:
		localel = Getelement()
		cellspace[localel].next = stack[s]
		stack[s] = localel
		cellspace[localel].discsize = i

def Init(s, n):
	Makenull(s)
	discctr = n
	while discctr >= 1:
		Push(discctr, s)
		discctr = discctr - 1

def Pop(s):
	global cellspace, freelist, stack
	if (stack[s] > 0):
		temp1 = cellspace[stack[s]].discsize
		temp = cellspace[stack[s]].next
		cellspace[stack[s]].next = freelist
		freelist = stack[s]
		stack[s] = temp
		return temp1
	else:
		Error("nothing to pop.")

def Move(s1, s2):
	global movesdone
	Push(Pop(s1), s2)
	movesdone = movesdone + 1

def tower(i, j, k):
	if k == 1:
		Move(i, j)
	else:
		other = 6 - i - j
		tower(i, other, k-1)
		Move(i, j)
		tower(other, j, k-1)

def Towers():
	global cellspace, freelist, maxcells, movesdone
	InitStack()
	InitCellspace()
	for i in range(maxcells):
		cellspace[i+1].next = i
	freelist = maxcells
	Init(1, 14)
	Makenull(2)
	Makenull(3)
	movesdone = 0
	tower(1, 2, 14)
	if movesdone != 16383:
		print "Error in Towers."

# ---------------------------- Puzzle ------------------------------------

# I am implementing this as a class in order to keep it organized.

class PuzzleClass:
	"Implements Puzzle Benchmark"
	
	#Constants
	size = 512         #Note: Defined as 511 in C
	classmax = 3
	typemax = 12
	d = 8
	
	def __init__(self):
		#Import class constants
		typemax = PuzzleClass.typemax
		size    = PuzzleClass.size
		d       = PuzzleClass.d

		#Add instance variables
		self.puzzl      = [true] * size
		self.piececount = [13, 3, 1, 1]
		self.zclass     = [None] * (typemax + 1)
		self.piecemax   = [None] * (typemax + 1)
		self.p          = MakeMatrix(typemax+1, size)
		self.n          = None
		self.kount      = 0
		
		#Additional initialization
		# Remember, range(1, 6) only gives [1,2,3,4,5]
		for i in range(1, 6):
			for j in range(1, 6):
				for k in range (1, 6):
					self.puzzl[i+d*(j+d*k)] = false
		for i in range(typemax+1):
			for m in range(size):
				self.p[i][m] = false

		self.InitPart( 0, 3, 1, 0, 0)
		self.InitPart( 1, 1, 0, 3, 0)
		self.InitPart( 2, 0, 3, 1, 0)
		self.InitPart( 3, 1, 3, 0, 0)
		self.InitPart( 4, 3, 0, 1, 0)
		self.InitPart( 5, 0, 1, 3, 0)
		self.InitPart( 6, 2, 0, 0, 1)
		self.InitPart( 7, 0, 2, 0, 1)
		self.InitPart( 8, 0, 0, 2, 1)
		self.InitPart( 9, 1, 1, 0, 2)
		self.InitPart(10, 1, 0, 1, 2)
		self.InitPart(11, 0, 1, 1, 2)
		self.InitPart(12, 1, 1, 1, 3) 

	def InitPart(self, index, maxI, maxJ, maxK, classVal):
		d = PuzzleClass.d
		for i in range(maxI+1):
			for j in range(maxJ+1):
				for k in range(maxK+1):
					self.p[index][i+d*(j+d*k)] = true
		self.zclass[index] = classVal
		self.piecemax[index] = maxI + maxJ*d + maxK*d*d

	def Run(self):
		m = 1+PuzzleClass.d*(1+PuzzleClass.d*1)
		if self.Fit(0, m):
			self.n = self.Place(0, m)
		else:
			print "Error1 in Puzzle"
		result = self.Trial(self.n)
		if not result:
			print "Error2 in Puzzle"
		elif self.kount != 2005:
			print "Error3 in Puzzle"

	def Fit(self, i, j):
		for k in range(self.piecemax[i]+1):
			if self.p[i][k]:
				if self.puzzl[j+k]:
					return false
		return true

	def Place(self, i, j):
		for k in range(self.piecemax[i]+1):
			if self.p[i][k]:
				self.puzzl[j+k] = true
		self.piececount[self.zclass[i]] = self.piececount[self.zclass[i]] - 1
		for k in range(j, PuzzleClass.size):
			if not self.puzzl[k]:
				return k
		return 0

	def Remove(self, i, j):
		for k in range(self.piecemax[i]+1):
			if self.p[i][k]:
				self.puzzl[j+k] = false
		self.piececount[self.zclass[i]] = self.piececount[self.zclass[i]] + 1

	def Trial(self, j):
		self.kount = self.kount + 1
		for i in range(PuzzleClass.typemax+1):
			if self.piececount[self.zclass[i]] != 0:
				if self.Fit(i, j):
					k = self.Place(i, j)
					if self.Trial(k) or (k == 0):
						return true
					else:
						self.Remove(i, j)
		return false

def Puzzle():
	PuzzleClass().Run()


# ----------------------------- FFT --------------------------------------

FFTSize = 256

def Exptab(n):
	# Vars
	#   double theta, divisor, h[26]
	#   int i, j, k, l, m
	#   complex e[n/2+1]
	
	theta = 3.1415926536
	divisor = 4.0
	h = [None] * 26
	e = [None] * (n / 2 + 2)

	for index in range(1, 26):
		h[index] = 1.0/(2.0*FFTCos(theta/divisor))
		divisor = divisor + divisor
	m = n / 2
	l = m / 2
	j = 1
	e[1] = 1.0+0.0j
	e[l+1] = 0.0+1.0j
	e[m+1] = -1.0+0.0j

	while 1:
		i = l / 2
		k = i
		while 1:
			e[k+1] = h[j]*(e[k+i+1]+e[k-i+1])
			k = k+l
			if k > m: 
				break
		j = min(j+1, 25)
		l = i
		if l <= 1: 
			break
	return e
	
def DoFFT(n, z, w, e, sqrinv):
	#Variables int n; complex z, w, e; double sqrinv
	#Locals int i, j, k, l, m
	
	m = n / 2
	l = 1
	
	while 1:
		k = 0
		j = l
		i = 1
		
		while 1:
			while 1:
				w[i+k] = z[i] + z[m+i]
				w[i+j] = e[k+1] * (z[i] - z[m+i])
				i = i + 1
				if i > j:
					break
			k = j
			j = k + l
			if j > m:
				break
		
		for index in range(1, n+1):
			z[index] = w[index]
		l = l + l     #l=l*2
		if l > m:
			break

	for i in range(1, n+1):
		z[i].real = sqrinv * z[i].real
		z[i].imag = -sqrinv * z[i].imag

def Random8K(aSeed):
	"Return a poor quality random number in the range of 8-8191"
	return (4855 * aSeed + 1731) & 8191

def FFTCos(x):
	"Computes cos of x (x in radians) by an expansion"
	result = 1.0
	factor = 1
	power = x
	for i in range(2, 11):
		factor = factor * i
		power = power * x
		if (i & 1) == 0:
			if (i & 3) == 0:
				result = result + power / factor
			else:
				result = result - power / factor
	return result

def FFT():
	"FFT Benchmark run function"
	e = Exptab(FFTSize)
	seed = 5767
	z = [ 0.0 ]
	for i in range(1, FFTSize+1):
		seed = Random8K(seed)
		seed1 = Random8K(seed)
		z.append(                                   \
		          (20.0 * (seed / 8192.0) - 10.0) + \
			  (20.0 * (seed1 / 8192.0) - 10.0)*1.0j )
		seed = seed1
	w = [ None ] * (FFTSize + 1)
	for i in range(20):
		DoFFT(FFTSize, z, w, e, 0.0625)
	if (   z[1].real < -9.3653 or z[1].real > -9.3652  \
	    or z[1].imag < -3.9869 or z[1].imag > -3.9868):
		print "Error in FFT"

# -------------------------- Run Benchmarks ------------------------------

def runBench(bench, name):
	"Run a benchmark with no arguments"
	starttime = time.clock()
	bench()
	print string.ljust(name, 16) + ":", int((time.clock() - starttime) * 1000), "ms"

def runBench1(bench, name, value):
	"Run a benchmark with one argument"
	starttime = time.clock()
	bench(value)
	print string.ljust(name, 16) + ":", int((time.clock() - starttime) * 1000), "ms"

def runNoInit(initFunction, bench, name):
	"Initialize the benchmark without counting the time, then run it."
	value = initFunction()
	runBench1(bench, name, value)


def go():
	runBench(Bubble, "Bubble")
	runBench(IntMM,  "IntMM")
	runBench(MM,     "MM")
	runBench(PermL,  "Perm")
	runBench(Queens, "Queens")
	runBench(Quick,  "Quick")
	runBench(Fast,   "Fast")
	runNoInit(FastInit, FastSort, "Fast no init")	
	runBench(Tree,   "Tree")
	runBench(Towers, "Towers")
	runBench(FFT,    "FFT")
	runBench(Puzzle, "Puzzle")

go()
